@@ -22,6 +22,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include <Pt.h>
 
 #include "ut_debugmsg.h"
 
@@ -30,7 +31,6 @@
 #include "xap_QNXApp.h"
 #include "xap_FakeClipboard.h"
 #include "gr_QNXImage.h"
-#include "xap_QNXFrame.h"
 #include "xap_QNXToolbar_Icons.h"
 #include "xap_QNX_TB_CFactory.h"
 #include "xap_Prefs.h"
@@ -44,18 +44,7 @@ XAP_QNXApp::XAP_QNXApp(XAP_Args * pArgs, const char * szAppName)
 
 	_setAbiSuiteLibDir();
 
-/*
-	// set some generic window sizes and positions
-	m_geometry.x = 1;
-	m_geometry.y = 1;
-	m_geometry.width = 600;
-	m_geometry.height = 600;
-
-	// by default, which will be applied if the user does NOT
-	// specify a --geometry argument, we only want to obey the
-	// size (which is set above), not a position.
-	m_geometry.flags = GEOMETRY_FLAG_SIZE;
-*/
+	memset(&m_geometry,0,sizeof(m_geometry));
 }
 
 XAP_QNXApp::~XAP_QNXApp(void)
@@ -81,7 +70,7 @@ bool XAP_QNXApp::initialize(void)
 void XAP_QNXApp::reallyExit(void)
 {
 	//There must be a nicer way to drop out of the event loop
-	PtExit(0);
+	PtQuitMainLoop();
 }
 
 UT_sint32 XAP_QNXApp::makeDirectory(const char *path, UT_sint32 mode) const
@@ -137,7 +126,27 @@ bool XAP_QNXApp::_loadFonts(void)
 	return true;
 }
 
-void XAP_QNXApp::_setAbiSuiteLibDir(void)
+void XAP_QNXApp::setWinGeometry(int x,int y,UT_uint32 width,UT_uint32 height, UT_uint32 flags)
+{
+	// TODO : do some range checking?
+ 	m_geometry.x = x;
+ 	m_geometry.y = y;
+ 	m_geometry.width = width;
+ 	m_geometry.height = height;
+ 	m_geometry.flags = flags;
+}
+
+void XAP_QNXApp::getWinGeometry(int *x,int *y,UT_uint32 *width,UT_uint32 *height,UT_uint32 *flags)
+{
+	UT_ASSERT(x && y && width && height);
+	*x = m_geometry.x;
+	*y = m_geometry.y;
+	*width = m_geometry.width;
+	*height = m_geometry.height;
+	*flags = m_geometry.flags;
+}
+
+void XAP_QNXApp::_setAbiSuiteLibDir()
 {
 	char buf[PATH_MAX];
 //	char buf2[PATH_MAX]; // not used?
@@ -180,15 +189,8 @@ void XAP_QNXApp::_setAbiSuiteLibDir(void)
 		return;
 	}
 
-	// TODO what to do ??  try the current directory...
-	
-	UT_DEBUGMSG(("ABISUITE_HOME not set and -lib not given.  Assuming current directory...."));
-
-	getcwd(buf,sizeof(buf));
-	int len = strlen(buf);
-	if (buf[len-1]=='/')				// trim trailing slash
-		buf[len-1] = 0;
-	XAP_App::_setAbiSuiteLibDir(buf);
+	// otherwise, use the hard-coded value
+	XAP_App::_setAbiSuiteLibDir(getAbiSuiteHome());
 	return;
 }
 
