@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Application Framework
  * Copyright (C) 1998 AbiSource, Inc.
  * 
@@ -32,10 +34,12 @@
 #include "xap_Dlg_MessageBox.h"
 #include "xap_UnixDlg_FileOpenSaveAs.h"
 #include "xap_UnixApp.h"
-#include "xap_UnixFrame.h"
+#include "xap_Frame.h"
+#include "xap_UnixFrameImpl.h"
 #include "xap_Strings.h"
 #include "xap_Prefs.h"
 #include "ut_debugmsg.h"
+#include "ut_string_class.h"
 
 #include "ut_png.h"
 #include "ut_svg.h"
@@ -44,11 +48,7 @@
 #include "fg_Graphic.h"
 #include "fg_GraphicRaster.h"
 
-#if defined(HAVE_GNOME)
-#include "gr_UnixGnomeImage.h"
-#else
 #include "gr_UnixImage.h"
-#endif
 
 #include <sys/stat.h>
 
@@ -69,7 +69,7 @@ XAP_Dialog * XAP_UnixDialog_FileOpenSaveAs::static_constructor(XAP_DialogFactory
 
 XAP_UnixDialog_FileOpenSaveAs::XAP_UnixDialog_FileOpenSaveAs(XAP_DialogFactory * pDlgFactory,
 														   XAP_Dialog_Id id)
-  : XAP_Dialog_FileOpenSaveAs(pDlgFactory,id), m_FS(0), m_preview(0)
+  : XAP_Dialog_FileOpenSaveAs(pDlgFactory,id), m_FS(0), m_preview(0), m_bExport(true)
 {
 	m_szFinalPathnameCandidate = NULL;
 }
@@ -111,12 +111,18 @@ static gint s_preview_exposed(GtkWidget * /* widget */,
 	return FALSE;
 }
 
+static void s_filetypechanged(GtkWidget * w, gpointer p)
+{
+	XAP_UnixDialog_FileOpenSaveAs * dlg = static_cast<XAP_UnixDialog_FileOpenSaveAs *>(p);
+	dlg->fileTypeChanged(w);
+}
+
 static gint
 fsel_key_event (GtkWidget *widget, GdkEventKey *event, XAP_Dialog_FileOpenSaveAs::tAnswer * answer)
 {
 #ifdef GDK_Escape
 	if (event->keyval == GDK_Escape) {
-		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
+		g_signal_emit_stop_by_name (G_OBJECT (widget), "key_press_event");
 		s_cancel_clicked ( widget, answer ) ;
 		return TRUE;
 	}
@@ -125,15 +131,13 @@ fsel_key_event (GtkWidget *widget, GdkEventKey *event, XAP_Dialog_FileOpenSaveAs
 	return FALSE;
 }
 
-static gint s_filename_select (GtkCList * w,
-			       gint row, gint col, GdkEvent * evt,
-			       gpointer ptr)
+static void file_selection_changed  (GtkTreeSelection  *selection,
+				     gpointer           ptr)
 {
   XAP_UnixDialog_FileOpenSaveAs * dlg = static_cast<XAP_UnixDialog_FileOpenSaveAs *> (ptr);
 
   UT_ASSERT(dlg);
   dlg->previewPicture();
-  return FALSE;
 }
 
 bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
@@ -141,7 +145,7 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 													 bool bCheckWritePermission,
 													 GtkWidget * filetypes_pulldown)
 {
-	GtkFileSelection * pFS = (GtkFileSelection *)pFSvoid;
+	GtkFileSelection * pFS = static_cast<GtkFileSelection *>(pFSvoid);
 
 	/*
 	  Run the dialog in a loop to catch bad filenames.
@@ -213,7 +217,7 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 			}
 
 			UT_cloneString(m_szFinalPathnameCandidate, szDialogFilename);
-			
+			if(szDialogFilename) FREEP(szDialogFilename);
 			// if we got here, the text wasn't a directory, so it's a file,
 			// and life is good
 			return (m_answer == a_OK);
@@ -239,14 +243,14 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 		// If, however, the user doesn't want suffixes, they don't have to have them.  
 
 		{
-			//UT_uint32 end = UT_pointerArrayLength((void **) m_szSuffixes);
+			//UT_uint32 end = UT_pointerArrayLength(static_cast<void **>(m_szSuffixes));
 
 			GtkWidget * activeItem = gtk_menu_get_active(GTK_MENU(
 				gtk_option_menu_get_menu(GTK_OPTION_MENU(filetypes_pulldown))));
 			UT_ASSERT(activeItem);
 
-			UT_sint32 nFileType = GPOINTER_TO_INT(gtk_object_get_user_data(
-				GTK_OBJECT(activeItem)));
+			UT_sint32 nFileType = GPOINTER_TO_INT(g_object_get_data(
+				G_OBJECT(activeItem), "user_data"));
 
 			// set to first item, which should probably be auto detect
 			// TODO : "probably" isn't very good.
@@ -266,7 +270,7 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 			
 			XAP_Prefs *pPrefs= pFrame->getApp()->getPrefs();
 
-			pPrefs->getPrefsValueBool((const XML_Char *)XAP_PREF_KEY_UseSuffix, &wantSuffix);
+			pPrefs->getPrefsValueBool(static_cast<const XML_Char *>(XAP_PREF_KEY_UseSuffix), &wantSuffix);
 
 			UT_DEBUGMSG(("UseSuffix: %d\n", wantSuffix));
 
@@ -286,7 +290,7 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 					UT_ASSERT(szSuffix);                            
 					UT_uint32 length = strlen(szDialogFilename) + strlen(szSuffix) + 1;
 					
-					szFinalPathname = (char *)UT_calloc(length,sizeof(char));
+					szFinalPathname = static_cast<char *>(UT_calloc(length,sizeof(char)));
 					
 					if (szFinalPathname)                            						
 						{                                               
@@ -415,36 +419,87 @@ ReturnTrue:
 bool XAP_UnixDialog_FileOpenSaveAs::_askOverwrite_YesNo(XAP_Frame * pFrame, const char * fileName)
 {
 	return (pFrame->showMessageBox(XAP_STRING_ID_DLG_OverwriteFile,
-										XAP_Dialog_MessageBox::b_YN,
-										XAP_Dialog_MessageBox::a_NO, // should this be YES?
-										fileName)
-						== XAP_Dialog_MessageBox::a_YES);
+				       XAP_Dialog_MessageBox::b_YN,
+				       XAP_Dialog_MessageBox::a_NO, // should this be YES?
+				       fileName)
+		== XAP_Dialog_MessageBox::a_YES);
 }
 	
 void XAP_UnixDialog_FileOpenSaveAs::_notifyError_OKOnly(XAP_Frame * pFrame, XAP_String_Id sid)
 {
 	pFrame->showMessageBox(sid,
-							XAP_Dialog_MessageBox::b_O,
-							XAP_Dialog_MessageBox::a_OK);
+			       XAP_Dialog_MessageBox::b_O,
+			       XAP_Dialog_MessageBox::a_OK);
 }
 
 void XAP_UnixDialog_FileOpenSaveAs::_notifyError_OKOnly(XAP_Frame * pFrame,
-														XAP_String_Id sid,
-														const char * sz1)
+							XAP_String_Id sid,
+							const char * sz1)
 {
 	pFrame->showMessageBox(sid,
-							XAP_Dialog_MessageBox::b_O,
-							XAP_Dialog_MessageBox::a_OK,
-							sz1);
+			       XAP_Dialog_MessageBox::b_O,
+			       XAP_Dialog_MessageBox::a_OK,
+			       sz1);
+}
+
+void XAP_UnixDialog_FileOpenSaveAs::fileTypeChanged(GtkWidget * w)
+{
+	if (!m_bExport)
+		return;
+
+	UT_sint32 nFileType = GPOINTER_TO_INT(g_object_get_data(
+				G_OBJECT(w), "user_data"));
+	UT_DEBUGMSG(("File type widget is %x filetype number is %d \n",w,nFileType));
+	if(nFileType == 0)
+	{
+		return;
+	}
+	UT_String sFileName = 	gtk_file_selection_get_filename(m_FS);
+	UT_String sSuffix = m_szSuffixes[nFileType-1];
+	sSuffix = sSuffix.substr(1,sSuffix.length()-1);
+	UT_uint32 i = 0;
+	bool bFoundComma = false;
+	for(i=0; i< sSuffix.length(); i++)
+	{
+		if(sSuffix[i] == ';')
+		{
+			bFoundComma = true;
+			break;
+		}
+	}
+	if(bFoundComma)
+	{
+		sSuffix = sSuffix.substr(0,i);
+	}
+//
+// Hard code a suffix
+//
+	if(strstr(sSuffix.c_str(),"gz") != NULL)
+	{
+		sSuffix = ".zabw";
+	}
+	bool bFoundSuffix = false;
+	for(i= sFileName.length()-1; i> 0; i--)
+	{
+		if(sFileName[i] == '.')
+		{
+			bFoundSuffix = true;
+			break;
+		}
+	}
+	if(!bFoundSuffix)
+	{
+		return;
+	}
+	sFileName = sFileName.substr(0,i);
+	sFileName += sSuffix;
+	gtk_file_selection_set_filename(m_FS,sFileName.c_str());
 }
 
 /*****************************************************************/
 
 void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 {
-	m_pUnixFrame = (XAP_UnixFrame *)pFrame;
-	UT_ASSERT(m_pUnixFrame);
-
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
 	// do we want to let this function handle stating the Unix
@@ -453,76 +508,74 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 
 	bool bCheckWritePermission = false;
 
-	const XML_Char * szTitle = NULL;
-	const XML_Char * szFileTypeLabel = NULL;
+	UT_UTF8String szTitle;
+	UT_UTF8String szFileTypeLabel;
 	switch (m_id)
 	{
-	case XAP_DIALOG_ID_INSERT_PICTURE:
-	  {
-	        szTitle = pSS->getValue(XAP_STRING_ID_DLG_IP_Title);
-		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
-		bCheckWritePermission = false;    
-	  }
-	case XAP_DIALOG_ID_FILE_OPEN:
-	{
-		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_OpenTitle);
-		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
-		bCheckWritePermission = false;
-		break;
-	}
-	case XAP_DIALOG_ID_FILE_IMPORT:
-	  {
-		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_ImportTitle);
-		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
-		bCheckWritePermission = false;
-	    break;
-	  }
-	case XAP_DIALOG_ID_INSERT_FILE:
-	  {
-		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_InsertTitle);
-		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
-		bCheckWritePermission = false;
-		break;
-	  }
-	case XAP_DIALOG_ID_FILE_SAVEAS:
-	{
-		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_SaveAsTitle);
-		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel);
-		bCheckWritePermission = true;
-		break;
-	}
-	case XAP_DIALOG_ID_FILE_EXPORT:
-	  {
-		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_ExportTitle);
-		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel);
-		bCheckWritePermission = true;
-	    break;
-	  }
-	case XAP_DIALOG_ID_PRINTTOFILE:
-	{
-		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_PrintToFileTitle);
-		szFileTypeLabel = pSS->getValue(XAP_STRING_ID_DLG_FOSA_FilePrintTypeLabel);
-		bCheckWritePermission = true;
-		break;
-	}
-	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-		break;
+		case XAP_DIALOG_ID_INSERT_PICTURE:
+			{
+				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_Title);
+				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+				bCheckWritePermission = false;    
+				break;
+			}
+		case XAP_DIALOG_ID_FILE_OPEN:
+			{
+				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_OpenTitle);
+				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+				bCheckWritePermission = false;
+				break;
+			}
+		case XAP_DIALOG_ID_FILE_IMPORT:
+			{
+				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_ImportTitle);
+				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+				bCheckWritePermission = false;
+				break;
+			}
+		case XAP_DIALOG_ID_INSERT_FILE:
+			{
+				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_InsertTitle);
+				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileOpenTypeLabel);
+				bCheckWritePermission = false;
+				break;
+			}
+		case XAP_DIALOG_ID_FILE_SAVEAS:
+			{
+				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_SaveAsTitle);
+				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel);
+				bCheckWritePermission = true;
+				break;
+			}
+		case XAP_DIALOG_ID_FILE_EXPORT:
+			{
+				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_ExportTitle);
+				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileSaveTypeLabel);
+				bCheckWritePermission = true;
+				break;
+			}
+		case XAP_DIALOG_ID_PRINTTOFILE:
+			{
+				szTitle = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_PrintToFileTitle);
+				szFileTypeLabel = pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FilePrintTypeLabel);
+				bCheckWritePermission = true;
+				break;
+			}
+		default:
+			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			break;
 	}
+	m_bExport = bCheckWritePermission;
 
 	// NOTE: we use our string mechanism to localize the dialog's
 	// NOTE: title and the error/confirmation message boxes.  we
 	// NOTE: let GTK take care of the localization of the actual
 	// NOTE: buttons and labels on the FileSelection dialog.
 	
-	GtkFileSelection *pFS = GTK_FILE_SELECTION(gtk_file_selection_new(szTitle));
+	GtkFileSelection *pFS = GTK_FILE_SELECTION(gtk_file_selection_new(szTitle.utf8_str()));
 	m_FS = pFS;
-	
-	connectFocus(GTK_WIDGET(pFS),pFrame);
-	gtk_window_set_modal (GTK_WINDOW (pFS), TRUE);
-	gtk_widget_show_all (GTK_WIDGET (pFS));
-	gtk_grab_add (GTK_WIDGET (pFS));
- 
+
+	abiSetupModalDialog(GTK_DIALOG(pFS), pFrame, this, GTK_RESPONSE_CANCEL);
 	GtkWidget * filetypes_pulldown = NULL;
 	
 	/*
@@ -547,27 +600,26 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		    gtk_widget_show (preview);
 		    m_preview = preview;
 
-		    GtkWidget * frame = gtk_frame_new (pSS->getValue(XAP_STRING_ID_DLG_IP_Activate_Label));
+		    GtkWidget * frame = gtk_frame_new (pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_Activate_Label).utf8_str());
+			gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 		    gtk_widget_show (frame);
 		    gtk_container_add (GTK_CONTAINER(frame), preview);
 
 		    gtk_box_pack_start(GTK_BOX(pulldown_hbox), frame, FALSE, TRUE, 0);
-		    gtk_widget_set_usize (frame, PREVIEW_WIDTH, PREVIEW_HEIGHT);
+		    gtk_widget_set_size_request (frame, PREVIEW_WIDTH, PREVIEW_HEIGHT);
 
 		    // the expose event off the preview
-		    gtk_signal_connect(GTK_OBJECT(preview),
+		    g_signal_connect(G_OBJECT(preview),
 				       "expose_event",
-				       GTK_SIGNAL_FUNC(s_preview_exposed),
+				       G_CALLBACK(s_preview_exposed),
 				       static_cast<gpointer>(this));
 
-		    gtk_signal_connect(GTK_OBJECT(pFS->file_list),
-				       "select-row",
-				       GTK_SIGNAL_FUNC(s_filename_select),
-				       static_cast<gpointer>(this));
+		    g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (pFS->file_list)), "changed", G_CALLBACK (file_selection_changed), this);
+
 		  }
 
 		// pulldown label
-		GtkWidget * filetypes_label = gtk_label_new(szFileTypeLabel);
+		GtkWidget * filetypes_label = gtk_label_new(szFileTypeLabel.utf8_str());
 		gtk_label_set_justify(GTK_LABEL(filetypes_label), GTK_JUSTIFY_RIGHT);
 		gtk_misc_set_alignment(GTK_MISC(filetypes_label), 1.0, 0.5);
 		gtk_widget_show(filetypes_label);
@@ -606,21 +658,21 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 			// Auto-detect is always an option, but a special one, so we use
 			// a pre-defined constant for the type, and don't use the user-supplied
 			// types yet.
-			g_snprintf(buffer, 1024, "%s", pSS->getValue(XAP_STRING_ID_DLG_FOSA_FileTypeAutoDetect));
+			g_snprintf(buffer, 1024, "%s", pSS->getValueUTF8(XAP_STRING_ID_DLG_FOSA_FileTypeAutoDetect).utf8_str());
 			thismenuitem = gtk_menu_item_new_with_label(buffer);
-			gtk_object_set_user_data(GTK_OBJECT(thismenuitem), GINT_TO_POINTER(XAP_DIALOG_FILEOPENSAVEAS_FILE_TYPE_AUTO));
+			g_object_set_data(G_OBJECT(thismenuitem), "user_data", GINT_TO_POINTER(XAP_DIALOG_FILEOPENSAVEAS_FILE_TYPE_AUTO));
 			gtk_widget_show(thismenuitem);
-			gtk_menu_append(GTK_MENU(menu), thismenuitem);
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), thismenuitem);
 
 			UT_uint32 activeItemIndex = 0;
 			
 			// add list items
 			{
-				UT_ASSERT(UT_pointerArrayLength((void **) m_szSuffixes) ==
-						  UT_pointerArrayLength((void **) m_szDescriptions));
+				UT_ASSERT(UT_pointerArrayLength(reinterpret_cast<void **>(const_cast<char **>(m_szSuffixes))) ==
+						  UT_pointerArrayLength(reinterpret_cast<void **>(const_cast<char **>(m_szDescriptions))));
 				
 				// measure one list, they should all be the same length
-				UT_uint32 end = UT_pointerArrayLength((void **) m_szDescriptions);
+				UT_uint32 end = UT_pointerArrayLength(reinterpret_cast<void **>(const_cast<char **>(m_szDescriptions)));
 			  
 				for (UT_uint32 i = 0; i < end; i++)
 				{
@@ -630,9 +682,15 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 					
 					g_snprintf(buffer, 1024, "%s", m_szDescriptions[i]);
 					thismenuitem = gtk_menu_item_new_with_label(buffer);
-					gtk_object_set_user_data(GTK_OBJECT(thismenuitem), GINT_TO_POINTER(m_nTypeList[i]));
+					g_object_set_data(G_OBJECT(thismenuitem), "user_data", GINT_TO_POINTER(m_nTypeList[i]));
 					gtk_widget_show(thismenuitem);
-					gtk_menu_append(GTK_MENU(menu), thismenuitem);
+					gtk_menu_shell_append(GTK_MENU_SHELL(menu), thismenuitem);
+//
+// Attach a callback when it is activated to change the file suffix
+//
+					g_signal_connect(G_OBJECT(thismenuitem), "activate",
+									 G_CALLBACK(s_filetypechanged),	
+									 reinterpret_cast<gpointer>(this));
 				}
 			}
 
@@ -642,7 +700,7 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 			
 			// add menu to the option menu widget
 			gtk_option_menu_set_menu(GTK_OPTION_MENU(filetypes_pulldown), menu);
-
+			m_wFileTypes_PullDown = filetypes_pulldown;
 			// dialog; open dialog always does auto-detect
 			// TODO: should this also apply to the open dialog?
 			if (m_id == XAP_DIALOG_ID_FILE_SAVEAS)
@@ -654,23 +712,18 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	}
 	
 	// connect the signals for OK and CANCEL and the requisite clean-close signals
-	gtk_signal_connect_after(GTK_OBJECT(pFS),
-							 "destroy",
-							 NULL,
-							 NULL);
-	gtk_signal_connect(GTK_OBJECT(pFS),
+	g_signal_connect(G_OBJECT(pFS),
 							 "delete_event",
-							 GTK_SIGNAL_FUNC(s_delete_clicked),
+							 G_CALLBACK(s_delete_clicked),
 							 &m_answer);
 
-	gtk_signal_connect (GTK_OBJECT (pFS),
+	g_signal_connect (G_OBJECT (pFS),
 			    "key_press_event",
-			    GTK_SIGNAL_FUNC(fsel_key_event), &m_answer);
-	gtk_signal_connect(GTK_OBJECT(pFS->ok_button), "clicked",
-					   GTK_SIGNAL_FUNC(s_ok_clicked), &m_answer);
-	gtk_signal_connect(GTK_OBJECT(pFS->cancel_button), "clicked",
-					   GTK_SIGNAL_FUNC(s_cancel_clicked), &m_answer);
-
+			    G_CALLBACK(fsel_key_event), &m_answer);
+	g_signal_connect(G_OBJECT(pFS->ok_button), "clicked",
+					   G_CALLBACK(s_ok_clicked), &m_answer);
+	g_signal_connect(G_OBJECT(pFS->cancel_button), "clicked",
+					   G_CALLBACK(s_cancel_clicked), &m_answer);
 	if (m_id == XAP_DIALOG_ID_FILE_OPEN || m_id == XAP_DIALOG_ID_INSERT_PICTURE || m_id == XAP_DIALOG_ID_FILE_EXPORT || m_id == XAP_DIALOG_ID_INSERT_FILE) // only hide the buttons if we're opening a file/picture
 	  gtk_file_selection_hide_fileop_buttons(pFS);
 
@@ -732,9 +785,9 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	}
 
 	// get top level window and its GtkWidget *
-	XAP_UnixFrame * frame = static_cast<XAP_UnixFrame *>(pFrame);
-	UT_ASSERT(frame);
-	GtkWidget * parent = frame->getTopLevelWindow();
+	XAP_UnixFrameImpl * pUnixFrameImpl = static_cast<XAP_UnixFrameImpl *>(pFrame->getFrameImpl());
+	UT_ASSERT(pUnixFrameImpl);
+	GtkWidget * parent = pUnixFrameImpl->getTopLevelWindow();
 	UT_ASSERT(parent);
 
 	// center it
@@ -757,14 +810,13 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 		// what a long ugly line of code
 		GtkWidget * activeItem = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(filetypes_pulldown))));
 		UT_ASSERT(activeItem);
-		m_nFileType = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(activeItem)));
+		m_nFileType = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(activeItem), "user_data"));
 	}
 			  
 	if(pFS && GTK_IS_WIDGET(pFS))
 	  gtk_widget_destroy (GTK_WIDGET(pFS));
 
 	FREEP(szPersistDirectory);
-	m_pUnixFrame = NULL;
 
 	return;
 }
@@ -779,18 +831,20 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
 	// attach and clear the area immediately
+#ifndef WITH_PANGO	
 	GR_UnixGraphics* pGr = new GR_UnixGraphics(m_preview->window, unixapp->getFontManager(), m_pApp);
-	pGr->clearArea(0, 0, m_preview->allocation.width, m_preview->allocation.height);
+#else
+	GR_UnixGraphics* pGr = new GR_UnixGraphics(m_preview->window, m_pApp);
+#endif
+	
+	pGr->clearArea(0, 0, pGr->tlu(m_preview->allocation.width), pGr->tlu(m_preview->allocation.height));
 
-	gchar * buf = gtk_file_selection_get_filename (m_FS);
+	const gchar * buf = gtk_file_selection_get_filename (m_FS);
 
 	GR_Font * fnt = pGr->findFont("Times New Roman", "normal", "", "normal", "", "12pt");
 	pGr->setFont(fnt);
 
-	const XML_Char * str = pSS->getValue(XAP_STRING_ID_DLG_IP_No_Picture_Label);
-	int len = strlen (str);
-	UT_UCSChar * ucstext = new UT_UCSChar [len + 1]; 
-	UT_UCS_strcpy_char (ucstext, str);
+	UT_UTF8String str(pSS->getValueUTF8(XAP_STRING_ID_DLG_IP_No_Picture_Label).utf8_str());
 
 	int answer = 0;
 
@@ -806,7 +860,11 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 
 	if (!buf)
 	  {
-	    pGr->drawChars (ucstext, 0, len, 12, 35);
+#ifndef WITH_PANGO 		  
+	    pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
+#else
+		pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
+#endif		
 	    goto Cleanup;
 	  }
 
@@ -814,12 +872,20 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 	struct stat st;
 	if (!stat (buf, &st)) {
 		if (!S_ISREG(st.st_mode)) {
-			pGr->drawChars (ucstext, 0, len, 12, 35);
+#ifndef WITH_PANGO 		  
+			pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
+#else
+			pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
+#endif		
 			goto Cleanup;
 		}
 	}
 	else {
-		pGr->drawChars (ucstext, 0, len, 12, 35);
+#ifndef WITH_PANGO 		  
+		pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
+#else
+		pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
+#endif		
 		goto Cleanup;
 	}
 
@@ -832,7 +898,11 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 	if ((errorCode != UT_OK) || !pIEG)
 	{
 		DELETEP(pBB);
-		pGr->drawChars (ucstext, 0, len, 12, 35);
+#ifndef WITH_PANGO 		  
+		pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
+#else
+		pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
+#endif		
 		goto Cleanup;
 	}
 
@@ -840,47 +910,47 @@ gint XAP_UnixDialog_FileOpenSaveAs::previewPicture (void)
 
 	if ((errorCode != UT_OK) || !pGraphic)
 	  {
-	    pGr->drawChars (ucstext, 0, len, 12, 35);
+#ifndef WITH_PANGO 		  
+		pGr->drawChars (str.ucs4_str().ucs4_str(), 0, str.size(), pGr->tlu(12), pGr->tlu(35));
+#else
+		pGr->drawCharsDirectly(str.ucs4_str().ucs4_str(),0,str.size(),pGr->tlu(12),pGr->tlu(35));
+#endif		
 	    goto Cleanup;
 	  }
 
 	if ( FGT_Raster == pGraphic->getType () )
 	{
-#if defined(HAVE_GNOME)
-		pImage = new GR_UnixGnomeImage(NULL,false);
-#else
 		pImage = new GR_UnixImage(NULL);
-#endif
+
 		UT_ByteBuf * png = static_cast<FG_GraphicRaster*>(pGraphic)->getRaster_PNG();
 		UT_PNG_getDimensions (png, iImageWidth, iImageHeight);
 
 		if (m_preview->allocation.width >= iImageWidth && m_preview->allocation.height >= iImageHeight)
 		  scale_factor = 1.0;
 		else
-		  scale_factor = MIN( (double) m_preview->allocation.width/iImageWidth,
-				      (double) m_preview->allocation.height/iImageHeight);
+		  scale_factor = MIN( static_cast<double>(m_preview->allocation.width)/iImageWidth,
+				      static_cast<double>(m_preview->allocation.height)/iImageHeight);
 		
-		scaled_width  = (int)(scale_factor * iImageWidth);
-		scaled_height = (int)(scale_factor * iImageHeight);
+		scaled_width  = static_cast<int>(scale_factor * iImageWidth);
+		scaled_height = static_cast<int>(scale_factor * iImageHeight);
 
 		pImage->convertFromBuffer(png, scaled_width, scaled_height);
 		
 		pGr->drawImage(pImage,
-			       (int)((m_preview->allocation.width  - scaled_width ) / 2),
-			       (int)((m_preview->allocation.height - scaled_height) / 2));
+			       pGr->tlu(static_cast<int>((m_preview->allocation.width  - scaled_width ) / 2)),
+			       pGr->tlu(static_cast<int>((m_preview->allocation.height - scaled_height) / 2)));
 		
 		answer = 1;
 	}
-	else // if ( FGT_Vector == pGraphic->getType () )
+	else
 	{
-	  //pImage = new GR_VectorImage(NULL);
+	  UT_ASSERT_NOT_REACHED ();
 	}
 
  Cleanup:
 	DELETEP(pImage);
 	DELETEP(pGr);
 	DELETEP(fnt);
-	DELETEPV(ucstext);
 	DELETEP(pIEG);
 	DELETEP(pGraphic);
 
