@@ -22,6 +22,7 @@
 
 #include "ie_types.h"
 #include "pd_Document.h"
+#include "ut_string_class.h"
 
 //////////////////////////////////////////////////////////////////
 
@@ -30,31 +31,38 @@ class GR_Graphics;
 class ABI_EXPORT AP_Convert
 {
  public:
-	AP_Convert(XAP_App *pApp);
+	AP_Convert(int verbose=1);
 	~AP_Convert(void);
 
-    void convertTo(const char * szSourceFilename,
-				   IEFileType sourceFormat,
-				   const char * szTargetFilename,
-				   IEFileType targetFormat);
+	void convertTo(const char * szSourceFilename,
+		       IEFileType sourceFormat,
+		       const char * szTargetFilename,
+		       IEFileType targetFormat);
 	
 	void convertTo(const char * szSourceFilename,
-				   const char * szTargetSuffix);
-
-	void convertTo(const char * szSourceFilename,
-				   const char * szSourceSuffix,
-				   const char * szTargetFormat);
+		       const char * szTargetSuffixOrFilename);
 
 	void convertToPNG ( const char * szSourceFileName ) ;
 
 	void setVerbose(int level);
-	XAP_App* getApp() { return m_pApp; }
+	void setMergeSource (const char * source);
 
 	void print(const char * file, GR_Graphics * pGraphics);
 
+	void setImpProps (const char * props) {
+		m_impProps = props;
+	}
+
+	void setExpProps (const char * props) {
+		m_expProps = props;
+	}
+
  private:
 	int m_iVerbose;
-	XAP_App *m_pApp;
+	UT_UTF8String m_mergeSource;
+
+	UT_UTF8String m_impProps;
+	UT_UTF8String m_expProps;
 };
 
 #endif /* AP_CONVERT_H */
