@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -39,9 +39,10 @@
 #include "fl_BlockLayout.h"
 #include "ap_Preview_Paragraph.h"
 #include "xad_Document.h"
+#include <fribidi.h>
 
 AP_Dialog_Lists::AP_Dialog_Lists(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
-:	XAP_Dialog_Modeless(pDlgFactory, id),
+:	XAP_Dialog_Modeless(pDlgFactory, id, "interface/dialoglists"),
 	m_pView(0),
 	m_Answer(a_CLOSE),
 	m_isListAtPoint(false),
@@ -99,7 +100,7 @@ AP_Dialog_Lists::~AP_Dialog_Lists(void)
 	// Maybe we need another constrcutor
 
 	DELETEP(m_pFakeAuto);
-	
+
 	UNREFP(m_pFakeDoc);
 }
 
@@ -129,13 +130,13 @@ void AP_Dialog_Lists::_createPreviewFromGC(GR_Graphics* gc,
 // free any attached preview
 
 	DELETEP(m_pListsPreview);
-	
+
 	m_pListsPreview = new AP_Lists_preview(gc, this);
 	UT_ASSERT(m_pListsPreview);
-	
+
 	m_pListsPreview->setWindowSize(width, height);
 	//
-	// Generate the fake layout pointers and autonum we need for the 
+	// Generate the fake layout pointers and autonum we need for the
 	// preview
 	//
 	generateFakeLabels();
@@ -151,12 +152,12 @@ void AP_Dialog_Lists::_createPreviewFromGC(GR_Graphics* gc,
  */
 void AP_Dialog_Lists::event_PreviewAreaExposed(void)
 {
-	if (m_pListsPreview) 
+	if (m_pListsPreview)
 	{
 		fillFakeLabels();
 		m_pListsPreview->draw();
 	}
-	else 
+	else
 	{
 		UT_ASSERT(0);
 	}
@@ -178,7 +179,7 @@ void AP_Dialog_Lists::StopList(void)
 	getView()->cmdStopList();
 }
 
-fl_AutoNum * AP_Dialog_Lists::getAutoNum(void)
+fl_AutoNum * AP_Dialog_Lists::getAutoNum(void) const
 {
 	return getBlock()->getAutoNum();
 }
@@ -186,7 +187,7 @@ fl_AutoNum * AP_Dialog_Lists::getAutoNum(void)
 /*!
  * Returns the block at the current point.
  */
-fl_BlockLayout * AP_Dialog_Lists::getBlock(void)
+fl_BlockLayout * AP_Dialog_Lists::getBlock(void) const
 {
 	return getView()->getCurrentBlock();
 }
@@ -194,7 +195,7 @@ fl_BlockLayout * AP_Dialog_Lists::getBlock(void)
 /*!
  * This is the local cache of the change number reported in the AV_View. Only
  * do an auto update if this number is different from the AV_View.
- */ 
+ */
 UT_uint32 AP_Dialog_Lists::getTick(void)
 {
 	return m_iLocalTick;
@@ -203,14 +204,14 @@ UT_uint32 AP_Dialog_Lists::getTick(void)
 /*!
  * This is the local cache of the change number reported in the AV_View. Only
  * do an auto update if this number is different from the AV_View.
- */ 
+ */
 void AP_Dialog_Lists::setTick(UT_uint32 iTick)
 {
 	m_iLocalTick = iTick;
 }
 
 /*!
- * This method Does the stuff requested on the "action" button, "Apply" in the 
+ * This method Does the stuff requested on the "action" button, "Apply" in the
  * Modeless dialog and "OK" in the Modal dialog.
  * Read comments with for all the stuff that can happen.
  */
@@ -226,7 +227,7 @@ void AP_Dialog_Lists::Apply(void)
 	{
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "Symbol");
 	}
-	else if(m_NewListType > DASHED_LIST)
+	else if(m_NewListType > DASHED_LIST && m_NewListType < OTHER_NUMBERED_LISTS)
 	{
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, _getDingbatsFontName());
 	}
@@ -246,10 +247,10 @@ void AP_Dialog_Lists::Apply(void)
 		if(m_OutProps.getItemCount() > 0)
 			m_OutProps.clear();
 		sprintf(szStart,"%d",m_iStartValue);
-		m_OutProps.addItem((void *) "start-value"); 
+		m_OutProps.addItem((void *) "start-value");
 		m_Output[0] = (XML_Char *) szStart;
 		m_OutProps.addItem((void *) m_Output[0].c_str());
-		m_OutProps.addItem((void *) "list-style"); 
+		m_OutProps.addItem((void *) "list-style");
 		m_Output[1] = getBlock()->getListStyleString(m_NewListType);
 		m_OutProps.addItem((void *) m_Output[1].c_str());
 		m_OutProps.addItem((void *) "list-delim");
@@ -274,6 +275,11 @@ void AP_Dialog_Lists::Apply(void)
  * This piece of code changes the list style at the current point to the
  * Style requested by the user.
  */
+	UT_Vector vBlock;
+	UT_uint32 i = 0;
+	getView()->getBlocksInSelection(&vBlock);
+	UT_uint32 count = vBlock.getItemCount();
+	getView()->cmdUnselectSelection();
 	if(m_bApplyToCurrent == true && m_isListAtPoint == true &&  m_NewListType != NOT_A_LIST)
 	{
 		getView()->changeListStyle(getAutoNum(),m_NewListType,m_iStartValue,(XML_Char *) m_pszDelim,(XML_Char *) m_pszDecimal, m_pszFont,m_fAlign,m_fIndent);
@@ -287,11 +293,15 @@ void AP_Dialog_Lists::Apply(void)
 /*!
  * This code stops the list at the current point.
  */
-	else if ( m_isListAtPoint == true &&  m_NewListType == NOT_A_LIST)
+	if ( m_isListAtPoint == true &&  m_NewListType == NOT_A_LIST)
 	{
-		if(getBlock()->isListItem() == true)
+		for(i=0;i < count; i++)
 		{
-			getBlock()->StopList();
+			fl_BlockLayout * pBlock = (fl_BlockLayout *) vBlock.getNthItem(i);
+			if(pBlock->isListItem() == true)
+			{
+				getView()->getDocument()->StopList(pBlock->getStruxDocHandle());
+			}
 		}
 		clearDirty();
 		return;
@@ -299,9 +309,9 @@ void AP_Dialog_Lists::Apply(void)
 /*!
  * Start new list. 4 Possibilities.
  * 1. If there is a list at the current point and the user choose no list, stop
- *    the list the current point. 
+ *    the list the current point.
  *
- * 2. start a new list with the properties given if there is not a 
+ * 2. start a new list with the properties given if there is not a
  * list at the current point.
  *
  * 3. Start a sublist at the current point if a list already exists there and
@@ -311,82 +321,108 @@ void AP_Dialog_Lists::Apply(void)
  *    has one item in it.
  */
 	if(m_bStartNewList == true)
-	{ 
-		if(m_isListAtPoint == true && m_NewListType == NOT_A_LIST)
+	{
+		for(i=0;i < count; i++)
 		{
+			fl_BlockLayout * pBlock = (fl_BlockLayout *) vBlock.getNthItem(i);
+			if(pBlock->isListItem() == true && m_NewListType == NOT_A_LIST)
+			{
 //
 // This stops the current list.
 //
-			if(getBlock()->isListItem() == true)
-			{
-				getBlock()->StopList();
+				if(pBlock->isListItem() == true)
+				{
+					getView()->getDocument()->StopList(pBlock->getStruxDocHandle());
+				}
 			}
-			clearDirty();
-			return;
-		}
-		else if ( m_isListAtPoint != true && m_NewListType != NOT_A_LIST )
-		{
+			else if ( pBlock->isListItem() == false && m_NewListType != NOT_A_LIST )
+			{
 //
 // This starts the new list
 //
-			getBlock()->getDocument()->disableListUpdates();
-			getBlock()->StartList(m_NewListType,m_iStartValue,m_pszDelim,m_pszDecimal,m_pszFont,m_fAlign,m_fIndent, 0,1); 
-			getBlock()->getDocument()->enableListUpdates();
-			getBlock()->getDocument()->updateDirtyLists();
-			clearDirty();
-			return;
-		}
-		else if( getAutoNum() && (getAutoNum()->getNumLabels() > 1) && m_NewListType != NOT_A_LIST )
-		{
+				pBlock->getDocument()->disableListUpdates();
+				if(i == 0)
+				{
+					pBlock->StartList(m_NewListType,m_iStartValue,m_pszDelim,m_pszDecimal,m_pszFont,m_fAlign,m_fIndent, 0,1);
+					pBlock->getDocument()->enableListUpdates();
+					pBlock->getDocument()->updateDirtyLists();
+				}
+				else
+				{
+					fl_BlockLayout * pBlock = (fl_BlockLayout *) vBlock.getNthItem(i);
+					fl_BlockLayout * rBlock = (fl_BlockLayout *) pBlock->getPrev();
+					if(rBlock != NULL)
+					{
+						pBlock->resumeList(rBlock);
+						pBlock->getDocument()->enableListUpdates();
+					}
+				}
+
+			}
+			else if( pBlock->getAutoNum() && (pBlock->getAutoNum()->getNumLabels() > 1) && m_NewListType != NOT_A_LIST )
+			{
 //
 // This starts a sublist.
 //
-			UT_uint32 curlevel = getBlock()->getLevel();
-			UT_uint32 currID = getBlock()->getAutoNum()->getID();
-			curlevel++;
-			getBlock()->getDocument()->disableListUpdates();
+				UT_uint32 curlevel = pBlock->getLevel();
+				UT_uint32 currID = pBlock->getAutoNum()->getID();
+				curlevel++;
+				pBlock->getDocument()->disableListUpdates();
 //
 // Need to update m_fAlign and m_fIndent to reflect the higher level of indentation.
 //
-			m_fAlign = m_fAlign + (float) LIST_DEFAULT_INDENT;
-			getBlock()->StartList(m_NewListType,m_iStartValue,m_pszDelim,m_pszDecimal,m_pszFont,m_fAlign,m_fIndent, currID,curlevel);
-			getBlock()->getDocument()->enableListUpdates();
-			getBlock()->getDocument()->updateDirtyLists();
-			clearDirty();
-			return;
-		}
-		else if( getAutoNum() && (getAutoNum()->getNumLabels() <= 1) && m_NewListType != NOT_A_LIST )
-		{
+				if(i == 0)
+				{
+					m_fAlign = m_fAlign + (float) LIST_DEFAULT_INDENT;
+					pBlock->StartList(m_NewListType,m_iStartValue,m_pszDelim,m_pszDecimal,m_pszFont,m_fAlign,m_fIndent, currID,curlevel);
+					pBlock->getDocument()->enableListUpdates();
+					pBlock->getDocument()->updateDirtyLists();
+				}
+				else
+				{
+					fl_BlockLayout * pBlock = (fl_BlockLayout *) vBlock.getNthItem(i);
+					fl_BlockLayout * rBlock = (fl_BlockLayout *) pBlock->getPrev();
+					if(rBlock != NULL)
+					{
+						pBlock->resumeList(rBlock);
+						pBlock->getDocument()->enableListUpdates();
+					}
+				}
+			}
+			else if( pBlock->getAutoNum() && (pBlock->getAutoNum()->getNumLabels() <= 1) && m_NewListType != NOT_A_LIST )
+			{
 //
 // The list at the current point only has one item which is the current paragraph.
 // We can't share an sdh amongst two autonum's so we can't start a sublist.
 // We'll change the list style instead.
 //
-			getView()->changeListStyle(getAutoNum(),m_NewListType,m_iStartValue,(XML_Char *) m_pszDelim,(XML_Char *) m_pszDecimal, m_pszFont,m_fAlign,m_fIndent);
-			if(getAutoNum() != NULL)
-			{
-				getAutoNum()->update(0);
+				getView()->changeListStyle(pBlock->getAutoNum(),m_NewListType,m_iStartValue,(XML_Char *) m_pszDelim,(XML_Char *) m_pszDecimal, m_pszFont,m_fAlign,m_fIndent);
+				if(pBlock->getAutoNum() != NULL)
+				{
+					pBlock->getAutoNum()->update(0);
+				}
+
 			}
-			clearDirty();
-			return;
 		}
+		clearDirty();
+		return;
 	}
 /*!
  * OK Attach the block at this point to the previous list of the same margin.
  */
 	if(m_bResumeList == true &&  m_isListAtPoint != true )
-	{ 
-		fl_BlockLayout * rBlock = getBlock()->getPreviousListOfSameMargin();
-		if(rBlock == NULL)
+	{
+		for(i=0;i < count; i++)
 		{
-			clearDirty();
-			return;
+			fl_BlockLayout * pBlock = (fl_BlockLayout *) vBlock.getNthItem(i);
+			fl_BlockLayout * rBlock = pBlock->getPreviousListOfSameMargin();
+			if(rBlock != NULL)
+			{
+				pBlock->resumeList(rBlock);
+				pBlock->getDocument()->enableListUpdates();
+				pBlock->getDocument()->updateDirtyLists();
+			}
 		}
-		getBlock()->resumeList(rBlock);
-		getBlock()->getDocument()->enableListUpdates();
-		getBlock()->getDocument()->updateDirtyLists();
-		clearDirty();
-		return;
 	}
 	clearDirty();
 }
@@ -400,6 +436,15 @@ void AP_Dialog_Lists::Apply(void)
  */
 void  AP_Dialog_Lists::fillUncustomizedValues(void)
 {
+	// if we can get the current font, we will use it where appropriate
+	// the "NULL" string does not work too well on Windows in numbered lists
+	const XML_Char** props_in = NULL;
+	const XML_Char * font_family;
+	if (getView()->getCharFormat(&props_in))
+		font_family = UT_getAttribute("font-family", props_in);
+	else
+		font_family =(const XML_Char *) "NULL";
+
 	if(m_NewListType == NOT_A_LIST)
 	{
 		UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L");
@@ -409,40 +454,47 @@ void  AP_Dialog_Lists::fillUncustomizedValues(void)
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
 		UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) ".");
 		m_iStartValue = 1;
-	}	       
+	}
 
 	if(m_iLevel <= 0)
 	{
 		m_iLevel = 1;
 	}
-	
+
 	UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L");
 	m_fAlign =  (float)(LIST_DEFAULT_INDENT * m_iLevel);
 	m_fIndent = (float)-LIST_DEFAULT_INDENT_LABEL;
 
 	if( m_NewListType == NUMBERED_LIST)
-	{   
-		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
+	{
+		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, font_family);
 		UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) ".");
 		m_iStartValue = 1;
 		UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L.");
 	}
 	else if( m_NewListType == LOWERCASE_LIST)
-	{   
-		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
+	{
+		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, font_family);
 		UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) ".");
 		m_iStartValue = 1;
 		UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L)");
 	}
 	else if( m_NewListType == UPPERCASE_LIST)
-	{   
-		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
+	{
+		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, font_family);
 		UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) ".");
 		m_iStartValue = 1;
 		UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L)");
 	}
+	else if( m_NewListType == HEBREW_LIST)
+	{
+		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, font_family);
+		UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) "");
+		m_iStartValue = 1;
+		UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L");
+	}
 	else if( m_NewListType < BULLETED_LIST)
-	{   
+	{
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
 		UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) ".");
 		m_iStartValue = 1;
@@ -453,7 +505,7 @@ void  AP_Dialog_Lists::fillUncustomizedValues(void)
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
 		UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) ".");
 		m_iStartValue = 0;
-	}	       
+	}
 	if(m_NewListType == BULLETED_LIST || m_NewListType == IMPLIES_LIST)
 	{
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "Symbol");
@@ -462,10 +514,13 @@ void  AP_Dialog_Lists::fillUncustomizedValues(void)
 	{
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
 	}
-	else if(m_NewListType > DASHED_LIST)
+	else if(m_NewListType > DASHED_LIST && m_NewListType < OTHER_NUMBERED_LISTS)
 	{
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, _getDingbatsFontName());
 	}
+
+	if(props_in)
+		free(props_in);
 }
 
 /*!
@@ -504,7 +559,9 @@ void  AP_Dialog_Lists::fillFakeLabels(void)
  * or font from their specified values. Here we override whatever was in them
  * to the values they SHOULD be given the list type.
  */
-	if(m_NewListType == BULLETED_LIST || m_NewListType == IMPLIES_LIST)
+	if(  m_NewListType == BULLETED_LIST ||
+		 m_NewListType == IMPLIES_LIST  ||
+		 m_NewListType == DASHED_LIST )
 	{
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "Symbol");
 		UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L");
@@ -514,13 +571,13 @@ void  AP_Dialog_Lists::fillFakeLabels(void)
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
 		UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L");
 	}
-	else if(m_NewListType > DASHED_LIST)
+	else if(m_NewListType > DASHED_LIST && m_NewListType < OTHER_NUMBERED_LISTS)
 	{
 		UT_XML_strncpy( (XML_Char *) m_pszFont, 80, _getDingbatsFontName());
 		UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L");
 	}
 /*!
- * OK fill the preview variables with what they need and load them into 
+ * OK fill the preview variables with what they need and load them into
  * the preview class.
  */
 	m_pFakeAuto->setListType(m_NewListType);
@@ -532,7 +589,7 @@ void  AP_Dialog_Lists::fillFakeLabels(void)
 
 /*!
  *
- * This routine generates it's own AutoNum's and Layout pointers 
+ * This routine generates it's own AutoNum's and Layout pointers
  * for use in the preview
  */
 void  AP_Dialog_Lists::generateFakeLabels(void)
@@ -548,7 +605,7 @@ void  AP_Dialog_Lists::generateFakeLabels(void)
 	{
 		DELETEP(m_pFakeLayout[i]);
 		m_pFakeSdh[i] = (PL_StruxDocHandle) fakeApp++;
-		m_pFakeLayout[i] = new fl_Layout((PTStruxType) 0 , (PL_StruxDocHandle) m_pFakeSdh[i] ); 
+		m_pFakeLayout[i] = new fl_Layout((PTStruxType) 0 , (PL_StruxDocHandle) m_pFakeSdh[i] );
 	}
 	//
 	// Now generate the AutoNum
@@ -557,11 +614,11 @@ void  AP_Dialog_Lists::generateFakeLabels(void)
 	UNREFP(m_pFakeDoc);
 	m_pFakeDoc = new PD_Document(fakeApp);
 	m_pFakeAuto = new fl_AutoNum(m_iID, 0, m_NewListType, m_newStartValue, m_pszDelim, m_pszDecimal, (PD_Document *) m_pFakeDoc);
-	m_pFakeAuto->insertFirstItem(m_pFakeSdh[0], NULL,1);
+	m_pFakeAuto->insertFirstItem(m_pFakeSdh[0], NULL,1,false);
 	m_pFakeLayout[0]->setAutoNum(m_pFakeAuto);
 	for(i=1; i<4; i++)
 	{
-		m_pFakeAuto->insertItem(m_pFakeSdh[i],m_pFakeSdh[i-1]);
+		m_pFakeAuto->insertItem(m_pFakeSdh[i],m_pFakeSdh[i-1],false);
 		m_pFakeLayout[i]->setAutoNum(m_pFakeAuto);
 	}
 }
@@ -579,7 +636,7 @@ UT_UCSChar * AP_Dialog_Lists::getListLabel(UT_sint32 itemNo)
 	{
 		return NULL;
 	}
-	UT_sint32 cnt = UT_MIN(UT_UCS_strlen(tmp),80);
+	UT_sint32 cnt = UT_MIN(UT_UCS4_strlen(tmp),80);
 	UT_sint32 i;
 	for(i =0; i<= cnt; i++)
 		lab[i] = *tmp++;
@@ -669,7 +726,7 @@ void AP_Dialog_Lists::fillDialogFromVector( UT_Vector * vp)
 }
 
 /*!
- * This method reads all the List info from the document at the curent point 
+ * This method reads all the List info from the document at the curent point
  * and loads it into the dialog member variables.
  */
 void AP_Dialog_Lists::fillDialogFromBlock(void)
@@ -784,13 +841,13 @@ void AP_Dialog_Lists::fillDialogFromBlock(void)
 	// Now Do the Attributes first
 	//
 	if(va.getItemCount()>0)
-	{	
+	{
 //  		i = findVecItem(&va,PT_STYLE_ATTRIBUTE_NAME);
 //  		if( i>= 0)
 //  		{
 //  			m_DocListType = getBlock()->getListTypeFromStyle( (const XML_Char *) va.getNthItem(i+1));
 //  		}
-//  		else 
+//  		else
 //  		{
 //  			m_DocListType = NUMBERED_LIST;
 //  		}
@@ -810,10 +867,10 @@ void AP_Dialog_Lists::fillDialogFromBlock(void)
 		m_iID = getAutoNum()->getID();
 		m_DocListType = getAutoNum()->getType();
 		UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) getAutoNum()->getDecimal());
-		
+
 	}
 	else
-	{	       
+	{
 		m_iID = 0;
 		m_DocListType = NOT_A_LIST;
 	}
@@ -841,7 +898,7 @@ void AP_Dialog_Lists::PopulateDialogData(void)
 		const UT_UCSChar * tmp1 =  getBlock()->getListLabel();
 		if(tmp1 != NULL)
 		{
-			UT_sint32 cnt = UT_MIN(UT_UCS_strlen(tmp1),80);
+			UT_sint32 cnt = UT_MIN(UT_UCS4_strlen(tmp1),80);
 			UT_sint32 i;
 			for(i =0; i<=cnt; i++)
 				m_curListLabel[i] = *tmp1++;
@@ -871,7 +928,7 @@ UT_uint32 AP_Dialog_Lists::getID(void)
 }
 
 /*!
- * This method returns the index to the value corresponding to the 
+ * This method returns the index to the value corresponding to the
  * key in this props vector
  */
 UT_sint32  AP_Dialog_Lists::findVecItem(UT_Vector * v, char * key)
@@ -881,13 +938,13 @@ UT_sint32  AP_Dialog_Lists::findVecItem(UT_Vector * v, char * key)
 }
 
 /*!
- * This method returns the index to the value corresponding to the 
+ * This method returns the index to the value corresponding to the
  * key in this props vector
  */
 UT_sint32  AP_Dialog_Lists::findVecItem(UT_Vector * v, const char * key)
 {
 	UT_sint32 i = v->getItemCount();
-	if(i < 0) 
+	if(i < 0)
 		return i;
 	UT_sint32 j;
 	const char * pszV = NULL;
@@ -918,7 +975,7 @@ void AP_Dialog_Lists::ConstructWindowName(void)
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 	XML_Char * tmp = NULL;
 	UT_uint32 title_width = 33;
-	UT_XML_cloneNoAmpersands(tmp, pSS->getValue(AP_STRING_ID_DLG_Lists_Title));
+	UT_XML_cloneNoAmpersands(tmp, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Title).utf8_str());
 	BuildWindowName((char *) m_WindowName,(char*)tmp,title_width);
 	FREEP(tmp);
 }
@@ -935,7 +992,7 @@ bool AP_Dialog_Lists::setView(FV_View * view)
 	return true;
 }
 
-FV_View * AP_Dialog_Lists::getView(void)
+FV_View * AP_Dialog_Lists::getView(void) const
 {
 	XAP_Frame * pFrame = getActiveFrame();
 	return (FV_View *) pFrame->getCurrentView();
@@ -958,17 +1015,18 @@ const XML_Char* AP_Dialog_Lists::_getDingbatsFontName() const
 AP_Lists_preview::AP_Lists_preview(GR_Graphics * gc, AP_Dialog_Lists * pLists)
 :	XAP_Preview(gc),
 	m_pLists(pLists),
+	m_pFont(NULL),
 	m_fAlign(0.0f),
 	m_fIndent(0.0f),
 	m_iLine_height(0),
 	m_bFirst(true)
 {
-	m_pszFont[0] = '\0';
 	m_iLine_pos[0] = 0;
 }
 
 AP_Lists_preview::~AP_Lists_preview()
 {
+	DELETEP(m_pFont);
 }
 
 AP_Dialog_Lists * AP_Lists_preview::getLists(void)
@@ -978,7 +1036,19 @@ AP_Dialog_Lists * AP_Lists_preview::getLists(void)
 
 void AP_Lists_preview::setData(XML_Char * pszFont,float fAlign,float fIndent)
 {
-	UT_XML_strncpy(m_pszFont, 80, (const XML_Char *) pszFont);
+	//
+	// we draw at 16 points in this preview
+	//
+	if(!pszFont || strcmp(pszFont,"NULL")== 0)
+	{
+		m_pFont = m_gc->findFont("Times New Roman", "normal", "", "normal", "", "16pt");
+	}
+	else
+	{
+		m_pFont = m_gc->findFont((char *)pszFont, "normal", "", "normal", "", "16pt");
+	}	
+	UT_ASSERT(m_pFont);
+	
 	m_fAlign = fAlign;
 	m_fIndent = fIndent;
 }
@@ -986,25 +1056,15 @@ void AP_Lists_preview::setData(XML_Char * pszFont,float fAlign,float fIndent)
 
 void AP_Lists_preview::draw(void)
 {
+	UT_return_if_fail(m_pFont);
+	m_gc->setFont(m_pFont);
+	
 	UT_RGBColor clrGrey = UT_RGBColor(128,128,128);
 	UT_RGBColor clrBlack = UT_RGBColor(0,0,0);
-	UT_sint32 iWidth = getWindowWidth();
-	UT_sint32 iHeight = getWindowHeight();
+	UT_sint32 iWidth = m_gc->tlu(getWindowWidth());
+	UT_sint32 iHeight = m_gc->tlu(getWindowHeight());
 	UT_UCSChar ucs_label[50];
-	//
-	// we draw at 16 points in this preview
-	//
-	GR_Font * font;
-	if(strcmp(m_pszFont,"NULL")== 0)
-	{
-		font = m_gc->findFont("Times New Roman", "normal", "", "normal", "", "16pt");
-	}
-	else
-	{
-		font = m_gc->findFont((char *)m_pszFont, "normal", "", "normal", "", "16pt");
-	}
 
-	m_gc->setFont(font);
 	UT_sint32 iDescent = m_gc->getFontDescent();
 	UT_sint32 iAscent = m_gc->getFontAscent();
 	UT_sint32 iFont = iDescent + iAscent;
@@ -1017,19 +1077,19 @@ void AP_Lists_preview::draw(void)
 		m_gc->clearArea(0, 0, iWidth, iHeight);
 	}
 	m_gc->setColor(clrBlack);
-	UT_sint32 yoff = 5 ;
-	UT_sint32 xoff = 5 ;
+	UT_sint32 yoff = m_gc->tlu(5) ;
+	UT_sint32 xoff = m_gc->tlu(5) ;
 	UT_sint32 i,ii,yloc,awidth,aheight,maxw;
 	UT_sint32 twidth =0;
 	UT_sint32 j,xy;
 	float z,fwidth;
 	// todo 6.5 should be the page width in inches
 	float pagew = 2.0;
-	aheight = 16;
-	fwidth = static_cast<float>(iWidth);
-	
-	z = (float)((fwidth - 2.0*static_cast<float>(xoff)) /pagew);
-        UT_sint32 indent = static_cast<UT_sint32>( z*(m_fAlign+m_fIndent));
+	aheight = m_gc->tlu(16);
+	fwidth = static_cast<float>(m_gc->tdu(iWidth));
+
+	z = (float)((fwidth - 2.0*static_cast<float>(m_gc->tdu(xoff))) /pagew);
+  UT_sint32 indent = m_gc->tlu(static_cast<UT_sint32>( z*(m_fAlign+m_fIndent)));
 
 	if(indent < 0)
 		indent = 0;
@@ -1039,18 +1099,20 @@ void AP_Lists_preview::draw(void)
 		UT_UCSChar * lv = getLists()->getListLabel(i);
 		UT_sint32 len =0;
 
-		if(lv != NULL) 
+		if(lv != NULL)
 		{
 			//
 			// This code is here because UT_UCS_copy_char is broken
 			//
-			len = UT_MIN(UT_UCS_strlen(lv),51);
+			len = UT_MIN(UT_UCS4_strlen(lv),51);
 			for(j=0; j<=len;j++)
 			{
 				ucs_label[j] = *lv++;
 			}
-			ucs_label[len] = NULL;
-			len = UT_UCS_strlen(ucs_label);
+
+			ucs_label[len] = 0;
+
+			len = UT_UCS4_strlen(ucs_label);
 			yloc = yoff + iAscent + (iHeight - 2*yoff -iFont)*i/4;
 			//    m_gc->drawChars(ucs_label,0,len,xoff+indent,yloc);
 			twidth = m_gc->measureString(ucs_label,0,len,NULL);
@@ -1062,16 +1124,18 @@ void AP_Lists_preview::draw(void)
 	// Work out where to put grey areas to represent text
 	//
 	UT_sint32 xx,yy;
-	if(maxw > 0) 
+	if(maxw > 0)
 		maxw++;
-	
+
         // UT_sint32 vspace = (iHeight - 2*yoff -iFont)*i/16;
-	z = (float)((fwidth - 2.0*static_cast<float>(xoff)) /(float)pagew);
-	UT_sint32 ialign = static_cast<UT_sint32>( z*m_fAlign);
+	z = (float)((fwidth - 2.0*static_cast<float>(m_gc->tdu(xoff))) /(float)pagew);
+	UT_sint32 ialign = m_gc->tlu(static_cast<UT_sint32>( z*m_fAlign));
+
 	xx = xoff + ialign;
 	xy = xoff + ialign;
+
 	if(xx < (xoff + maxw + indent))
-		xy = xoff + maxw + indent + 1;
+		xy = xoff + maxw + indent + m_gc->tlu(1);
 	ii = 0;
 
 	for(i=0; i<4; i++)
@@ -1079,13 +1143,16 @@ void AP_Lists_preview::draw(void)
 		yloc = yoff + iAscent + (iHeight - 2*yoff -iFont)*i/4;
 		for(j=0; j< 2; j++)
 		{
-			yy = yloc + 5 + j*21;
+			yy = yloc + m_gc->tlu(5) + j*m_gc->tlu(21);
 			m_iLine_pos[ii++] = yy;
 		}
 	}
 	//
 	// Now finally draw the preview
 	//
+
+	FriBidiCharType iDirection = getLists()->getBlock()->getDominantDirection();
+
 	for(i=0; i<8; i++)
 	{
 		//
@@ -1099,37 +1166,80 @@ void AP_Lists_preview::draw(void)
 			//
 			UT_UCSChar * lv = getLists()->getListLabel(i/2);
 			UT_sint32 len =0;
-			
-			if(lv != NULL) 
+
+			if(lv != NULL)
 			{
-				len = UT_MIN(UT_UCS_strlen(lv),51);
-				for(j=0; j<=len;j++)
+				len = UT_MIN(UT_UCS4_strlen(lv),51);
+
+				if(len > 1 && XAP_App::getApp()->theOSHasBidiSupport() == XAP_App::BIDI_SUPPORT_GUI)
 				{
-					ucs_label[j] = *lv++;
+					FriBidiChar * fLogStr = new FriBidiChar[len+1];
+					FriBidiChar * fVisStr = new FriBidiChar[len+1];
+					UT_ASSERT(fLogStr && fVisStr);
+
+					for(j=0; j<=len;j++)
+						fLogStr[j] = lv[j];
+
+					fribidi_log2vis(/* input */
+			    		 fLogStr,
+					     len,
+					     &iDirection,
+					     /* output */
+					     fVisStr,
+					     NULL,
+					     NULL,
+					     NULL
+					     );
+
+					for(j=0; j<=len;j++)
+						ucs_label[j] = (UT_UCSChar)fVisStr[j];
+
+					delete [] fLogStr;
+					delete [] fVisStr;
 				}
-				ucs_label[len] = NULL;
-				len = UT_UCS_strlen(ucs_label);
+				else
+				{
+					for(j=0; j<=len;j++)
+						ucs_label[j] = *lv++;
+				}
+
+				ucs_label[len] = 0;
+				len = UT_UCS4_strlen(ucs_label);
 				yloc = yoff + iAscent + (iHeight - 2*yoff -iFont)*i/8;
-				m_gc->drawChars(ucs_label,0,len,xoff+indent,yloc);
+
+				if(iDirection == FRIBIDI_TYPE_RTL)
+					m_gc->drawChars(ucs_label,0,len,iWidth - xoff - indent - maxw,yloc);
+				else
+					m_gc->drawChars(ucs_label,0,len,xoff+indent,yloc);
+
 				yy = m_iLine_pos[i];
 				awidth = iWidth - 2*xoff - xy;
-				m_gc->fillRect(clrGrey,xy,yy,awidth,aheight);
+
+				if(iDirection == FRIBIDI_TYPE_RTL)
+					m_gc->fillRect(clrGrey,xoff,yy,awidth,aheight);
+				else
+					m_gc->fillRect(clrGrey,xy,yy,awidth,aheight);
 			}
 			else
 			{
 				yy = m_iLine_pos[i];
 				awidth = iWidth - 2*xoff - xy;
-				m_gc->fillRect(clrGrey,xy,yy,awidth,aheight);
+
+				if(iDirection == FRIBIDI_TYPE_RTL)
+					m_gc->fillRect(clrGrey,xoff,yy,awidth,aheight);
+				else
+					m_gc->fillRect(clrGrey,xy,yy,awidth,aheight);
 			}
 		}
 		else
 		{
 			yy = m_iLine_pos[i];
 			awidth = iWidth - 2*xoff - xx;
-			m_gc->fillRect(clrGrey,xx,yy,awidth,aheight);
+
+			if(iDirection == FRIBIDI_TYPE_RTL)
+				m_gc->fillRect(clrGrey,xoff,yy,awidth,aheight);
+			else
+				m_gc->fillRect(clrGrey,xy,yy,awidth,aheight);
 		}
 	}
-
-	delete font;
 }
-
