@@ -39,15 +39,23 @@
 #include "ap_TopRuler.h"		// for AP_TopRulerInfo
 
 AP_Dialog_Tab::AP_Dialog_Tab(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
-	: XAP_Dialog_NonPersistent(pDlgFactory,id), m_answer(a_OK), m_pFrame(0),
-	  m_pCallbackFn(0), m_closure(0)
+	: XAP_Dialog_NonPersistent(pDlgFactory,id, "interface/dialogtabs"),
+	  m_answer(a_OK), 
+	  m_pFrame(0),
+	  m_dim(DIM_IN),
+	  m_pszTabStops(0),
+	  m_pCallbackFn(0), 
+	  m_closure(0)
 {
+  m_pszTabStops = new char [1]; m_pszTabStops[0] = 0;
 }
 
 AP_Dialog_Tab::~AP_Dialog_Tab(void)
 {
-	delete [] m_pszTabStops;
-	UT_VECTOR_PURGEALL(fl_TabStop *, m_tabInfo);
+  if (NULL != m_pszTabStops)
+    delete [] m_pszTabStops;
+
+  UT_VECTOR_PURGEALL(fl_TabStop *, m_tabInfo);
 }
 
 void AP_Dialog_Tab::setSaveCallback (TabSaveCallBack pCb, void * closure)
@@ -196,8 +204,11 @@ void AP_Dialog_Tab::_event_TabSelected( UT_sint32 index )
 void AP_Dialog_Tab::_event_Set(void)
 {
 	UT_String buffer;
-	
+
 	buildTab(buffer);
+
+	UT_DEBUGMSG(("DOM: %s\n", buffer.c_str()));
+
 	const char *cbuffer = buffer.c_str();
 	int Dimension_size = 0;
 	while(cbuffer[Dimension_size] != 0)
@@ -245,7 +256,7 @@ void AP_Dialog_Tab::_event_Set(void)
 
 	UT_ASSERT(m_pFrame); // needs to be set from runModal for some of the event_'s to work
 
-	FV_View *pView = (FV_View *)m_pFrame->getCurrentView();
+	FV_View *pView = static_cast<FV_View *>(m_pFrame->getCurrentView());
 
 	buildTabStops(pView->getGraphics(), m_pszTabStops, m_tabInfo);
 
@@ -255,7 +266,7 @@ void AP_Dialog_Tab::_event_Set(void)
 
 	for (i = 0; i < m_tabInfo.getItemCount(); i++ )
 	{
-		fl_TabStop *pTabInfo = (fl_TabStop *)m_tabInfo.getNthItem(i);
+		fl_TabStop *pTabInfo = static_cast<fl_TabStop *>(m_tabInfo.getNthItem(i));
 		UT_ASSERT(pTabInfo);
 
 		// if we have a tab at that unit
@@ -280,13 +291,13 @@ void AP_Dialog_Tab::_event_Clear(void)
 
 	if(index != -1)
 	{
-		UT_ASSERT(index < (UT_sint32)m_tabInfo.getItemCount());
+		UT_return_if_fail(index < static_cast<UT_sint32>(m_tabInfo.getItemCount()));
 
-		_deleteTabFromTabString((fl_TabStop *)m_tabInfo.getNthItem(index));
+		_deleteTabFromTabString(static_cast<fl_TabStop *>(m_tabInfo.getNthItem(index)));
 
-		UT_ASSERT(m_pFrame); // needs to be set from runModal for some of the event_'s to work
+		UT_return_if_fail(m_pFrame); // needs to be set from runModal for some of the event_'s to work
 
-		FV_View *pView = (FV_View *)m_pFrame->getCurrentView();
+		FV_View *pView = static_cast<FV_View *>(m_pFrame->getCurrentView());
 
 		buildTabStops(pView->getGraphics(), m_pszTabStops, m_tabInfo);
 
@@ -311,12 +322,13 @@ void AP_Dialog_Tab::_event_ClearAll(void)
 {
 	UT_DEBUGMSG(("AP_Dialog_Tab::_event_ClearAll\n"));
 
-	UT_ASSERT(m_pFrame); // needs to be set from runModal for some of the event_'s to work
-
-	FV_View *pView = (FV_View *)m_pFrame->getCurrentView();
+	UT_return_if_fail(m_pFrame); // needs to be set from runModal for some of the event_'s to work
 
 	delete [] m_pszTabStops;
 	m_pszTabStops = new char [1]; m_pszTabStops[0] = 0;
+
+	FV_View *pView = static_cast<FV_View *>(m_pFrame->getCurrentView());
+
 	buildTabStops(pView->getGraphics(), m_pszTabStops, m_tabInfo);
 
 	_clearList();
@@ -400,32 +412,23 @@ void AP_Dialog_Tab::clearList()
 }
 
 
-void AP_Dialog_Tab::buildTab( UT_String &buffer )
+void AP_Dialog_Tab::buildTab( UT_String & buffer )
 {
 	// get current value from member
 	const XML_Char* szOld = _gatherTabEdit();
 	const XML_Char* szNew = UT_reformatDimensionString(m_dim, szOld); 
 
 	UT_String_sprintf( buffer, "%s/%c%c", szNew, AlignmentToChar(_gatherAlignment()),
-		 				((char)_gatherLeader())+'0');
+		 				(static_cast<char>(_gatherLeader()))+'0');
 }
-/*void AP_Dialog_Tab::buildTab( UT_String *buffer )
-{
-	// get current value from member
-	const XML_Char* szOld = _gatherTabEdit();
-	const XML_Char* szNew = UT_reformatDimensionString(m_dim, szOld); 
-
-	UT_String_sprintf( *buffer, "%s/%c%c", szNew, AlignmentToChar(_gatherAlignment()),
-		 				((char)_gatherLeader())+'0');
-}*/
 
 void AP_Dialog_Tab::_event_somethingChanged()
 {
 	UT_String buffer;
 
 	buildTab( buffer );
-
-	UT_DEBUGMSG(("AP_Dialog_Tab::_event_somethingChanged  [%s]\n", buffer.c_str() ));
+	const char *cbuffer = buffer.c_str();
+	UT_DEBUGMSG(("AP_Dialog_Tab::_event_somethingChanged  [%s]\n", cbuffer ));
 
 	// check to see if the current tab is in the list
 	bool bEnableClear = false;
@@ -442,9 +445,9 @@ void AP_Dialog_Tab::_event_somethingChanged()
 
 	for ( UT_uint32 i = 0; i < m_tabInfo.getItemCount(); i++ )
 	{
-		fl_TabStop *pTabInfo = (fl_TabStop *)m_tabInfo.getNthItem(i);
+		fl_TabStop *pTabInfo = static_cast<fl_TabStop *>(m_tabInfo.getNthItem(i));
 		UT_ASSERT(pTabInfo);
-		const char *cbuffer = buffer.c_str();
+
 		// if we have a tab at that unit
 		if ( !strcmp(cbuffer, _getTabString(pTabInfo)) )
 		{
@@ -470,7 +473,7 @@ char *AP_Dialog_Tab::_getTabDimensionString(UT_uint32 tabIndex)
 
 	UT_ASSERT(tabIndex < m_tabInfo.getItemCount());
 
-	fl_TabStop *pTabInfo = (fl_TabStop *)m_tabInfo.getNthItem(tabIndex);
+	fl_TabStop *pTabInfo = static_cast<fl_TabStop *>(m_tabInfo.getNthItem(tabIndex));
 
 	const char* pStart = &m_pszTabStops[pTabInfo->getOffset()];
 	const char* pEnd = pStart;
@@ -619,7 +622,7 @@ void AP_Dialog_Tab::_doSpin(tControl id, UT_sint32 amt)
 	}
 
 	// value is now in desired units, so change it
-	d += (dSpinUnit * (double) amt);
+	d += (dSpinUnit * static_cast<double>(amt));
 	if (d < dMin)
 		d = dMin;
 
