@@ -1,19 +1,19 @@
 /* AbiWord
- * Copyright (C) 1998-2001 AbiSource, Inc.
- * 
+ * Copyright (C) 1998-2000 AbiSource, Inc.
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -53,14 +53,14 @@
 #define _CT		EV_EMC_TEXT
 #define _CM		EV_EMC_MISSPELLEDTEXT
 #define _CL		EV_EMC_LEFTOFTEXT
-
-#ifdef BIDI_ENABLED
 #define _CR		EV_EMC_RIGHTOFTEXT
-#endif
-
 #define _CI		EV_EMC_IMAGE
 #define _CZ		EV_EMC_IMAGESIZE
 #define _CF		EV_EMC_FIELD
+#define _CH		EV_EMC_HYPERLINK
+#define _CV		EV_EMC_REVISION
+#define _CTV		EV_EMC_VLINE
+#define _CTH		EV_EMC_HLINE
 
 #define _B0		| EV_EMB_BUTTON0
 #define _B1		| EV_EMB_BUTTON1
@@ -69,6 +69,8 @@
 #define _B4		| EV_EMB_BUTTON4
 #define _B5		| EV_EMB_BUTTON5
 
+#define DO_NOT_USE ""
+
 ap_bs_Mouse MouseTable[] =
 {
 
@@ -76,7 +78,7 @@ ap_bs_Mouse MouseTable[] =
 	// TODO should be and some are just for testing until
 	// TODO the full features are implemented.  some i've
 	// TODO just filled in some guesses, but never used.
-	
+
 //	Button-0 (no buttons pressed)
 //  { context	{ click	dblclick	drag,				dbldrag,	release,	doublerelease	}},
 	{_CU _B0,	{ "",	"",			"cursorDefault",	"",			"",			""				}},
@@ -86,36 +88,52 @@ ap_bs_Mouse MouseTable[] =
 	{_CI _B0,	{ "",	"",			"cursorImage",		"",			"",			""				}},
 	{_CZ _B0,	{ "",	"",			"cursorImageSize",	"",			"",			""				}},
 	{_CF _B0,	{ "",	"",			"cursorDefault",	"",			"",			""				}},
-#ifdef BIDI_ENABLED
-	{_CR _B0,	{ "",	"",			"cursorLeftArrow",	"",			"",			""				}},
-#endif
-//	Button-1, Unknown-context
-//  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease }},
-	{_CU _B1,	{ "warpInsPtToXY",	"",				"",			"",				"",			""	}},
+	{_CR _B0,	{ "",	"",			"cursorDefault",	"",			"",			""				}},
+	{_CH _B0,	{ "",	"",			"hyperlinkStatusBar",	"",			"",			"",				}},
+	{_CTV _B0,	{ "",	"",			"cursorVline",	"",			"",			"",				}},
+	{_CTH _B0,	{ "",	"",			"cursorHline",	"",			"",			"",				}},
 
+
+//	Button-1, Image-context
+//  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease }},
+	{_CI _B1,	{ "selectObject",	"selectObject",	"",			"",				"",			""	}},
+	{_CI _B1 _C,{ "",				"",				"dragImage","",				"dropImage",""	}},
+	{_CF _B1,	{ "warpInsPtToXY",	"selectObject",	"",			"",				"",			""	}},
+	{_CF _B1,	{ "warpInsPtToXY",	"selectObject",	"",			"",				"",			""	}},
+	
 //	Button-1, Text-context
 //  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease	}},
 	{_CT _B1,	{ "warpInsPtToXY",	"selectWord",	"dragToXY",	"dragToXYword",	"endDrag",	"endDrag"		}},
 	{_CT _B1 _S,{ "extSelToXY",		"",				"dragToXY", "",				"endDrag",	"endDrag"		}},
-	{_CT _B1 _C,{ "selectWord",		"",				"",			"",				"endDrag",	"endDrag"		}},
-	
+	{_CT _B1 _C,{ "",		"",		       	"dragSelectionBegin",	       	"dragSelectionBegin",		        "dragSelectionEnd",	"dragSelectionEnd"		}},
+
+
+//	Button-1, Left-of-Text-context (left-margin)
+//  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease	}},
+	{_CL _B1,	{ "selectLine",		"selectBlock",	"dragToXY",	"dragToXYword",	"endDrag",	"endDrag"		}},
+
 //	Button-1, Misspelled-Word-Text-context
 //  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease	}},
 	{_CM _B1,	{ "warpInsPtToXY",	"selectWord",	"dragToXY",	"dragToXYword",	"endDrag",	"endDrag"		}},
 	{_CM _B1 _S,{ "extSelToXY",		"",				"dragToXY", "",				"endDrag",	"endDrag"		}},
 	{_CM _B1 _C,{ "selectWord",		"",				"",			"",				"endDrag",	"endDrag"		}},
-
-//	Button-1, Left-of-Text-context (left-margin)
-//  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease	}},
-	{_CL _B1,	{ "selectLine",		"selectBlock",	"dragToXY",	"dragToXYword",	"endDrag",	"endDrag"		}},
-#ifdef BIDI_ENABLED
 	{_CR _B1,	{ "selectLine",		"selectBlock",	"dragToXY",	"dragToXYword",	"endDrag",	"endDrag"		}},
-#endif
+
+//  button-1, hyperlink context
+	{_CH _B1,	{ "warpInsPtToXY",	"hyperlinkJump",	"dragToXY",	"dragToXYword",	"endDrag",	"endDrag"	}},
+	{_CV _B1,	{ "warpInsPtToXY",	"selectWord",	"dragToXY",	"dragToXYword",	"endDrag",	"endDrag"		}},
 
 //	Button-1, Image-context
 //  { context	{ click				doubleclick		drag,		dbldrag,	release,		doublerelease	}},
+	{_CZ _B1,	{ "",				"",				"resizeImage",	"",		"endResizeImage",	""				}},
 //	{_CI _B1,	{ "selectImage",	"editImage",	"moveImage","",			"endImageMove",	""				}},
 
+//	Button-1, TableLine-context
+
+	{_CTV _B1,	{ "beginVDrag",		 "clearSetCols", "dragVline",	"",		"endDragVline",	""				}},
+	{_CTH _B1,	{ "beginHDrag",		 "clearSetRows", "dragHline",	"",		"endDragHline",	""				}},
+
+
 //	Button-1, ImageSize-context
 //  { context	{ click				doubleclick		drag,		dbldrag,	release,		doublerelease	}},
 //	{_CI _B1,	{ "startImageSize",	"",				"dragImageSize","",		"endImageSize",	""				}},
@@ -127,9 +145,11 @@ ap_bs_Mouse MouseTable[] =
 //	Button-2, Text-context
 //  { context	{ click				doubleclick		drag,		dbldrag,		release,	doublerelease	}},
 
+#ifdef XP_UNIX_TARGET_GTK
 // special key binding for X11-style middle mouse selection (not to be confused with the clipboard)
 	{_CT _B2,	{ "pasteSelection",	"",				"",			"",				"",			""				}},
-
+	{_CH _B2,	{ "pasteSelection",	"",				"",			"",				"",			""				}},
+#endif
 
 //	Button-3, context menus
 //  { context	{ click					dblclick	drag,	dbldrag,	release,	doublerelease	}},
@@ -137,30 +157,51 @@ ap_bs_Mouse MouseTable[] =
 	{_CT _B3,	{ "contextText",		"",			"",		"",			"",			""				}},
 	//{_CL _B3,	{ "contextLeftOfText",	"",			"",		"",			"",			""				}},
 	{_CM _B3,	{ "contextMisspellText","",			"",		"",			"",			""				}},
-	//{_CI _B3,	{ "contextImage",		"",			"",		"",			"",			""				}},
+	{_CI _B3,	{ "contextImage",		"",			"",		"",			"",			""				}},
 	//{_CZ _B3,	{ "contextImageSize",	"",			"",		"",			"",			""				}},
 	//{_CF _B3,	{ "contextField",		"",			"",		"",			"",			""				}},
 
+	{_CH _B3,	{ "contextHyperlink",		"",			"",		"",			"",			""				}},
+
+	{_CV _B3,	{ "contextRevision",		"",			"",		"",			"",			""				}},
+
 //  Button-4, Mouse wheel (3 lines up)... yes, a hack.
 	{_CU _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CU _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
 	{_CT _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CT _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
 	{_CL _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
-#ifdef BIDI_ENABLED
+	{_CL _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
 	{_CR _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
-#endif	
 	{_CM _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CM _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
 	{_CI _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CI _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
 	{_CZ _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CZ _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
 	{_CF _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CF _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
+	{_CH _B4,	{ "scrollWheelMouseUp",		"scrollWheelMouseUp",			"",		"",			"",			""				}},
+	{_CH _B4 _C,{ "zoomIn",		            "zoomIn",           			"",		"",			"",			""				}},
+
 
 //  Button-5, Mouse wheel (3 lines down)
 	{_CU _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CU _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 	{_CT _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CT _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 	{_CL _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CL _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 	{_CM _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CM _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 	{_CI _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CI _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 	{_CZ _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CZ _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 	{_CF _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CF _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
+	{_CH _B5,	{ "scrollWheelMouseDown",		"scrollWheelMouseDown",			"",		"",			"",			""				}},
+	{_CH _B5 _C,{ "zoomOut",		            "zoomOut",           			"",		"",			"",			""				}},
 
 #if defined(PT_TEST) || defined(FMT_TEST) || defined(UT_TEST)
 //  { context		{ click			doubleclick		drag,		dbldrag,	release,	doublerelease	}},
@@ -187,21 +228,21 @@ DEFLEN(MouseTable);
 
 ap_bs_NVK NVKTable[] =
 {
-//	{nvk,				{ none,					_S,					_C,				_S_C,		
+//	{nvk,				{ none,					_S,					_C,				_S_C,
 //  					  _A,					_A_S,				_A_C,			_A_C_S				}},
-	{EV_NVK_BACKSPACE,	{ "delLeft", 			"delLeft",			"delBOW",		"",			
+	{EV_NVK_BACKSPACE,	{ "delLeft", 			"delLeft",			"delBOW",		"",
 						  "",					"",					"",				""					}},
 	{EV_NVK_SPACE,		{ "insertSpace",		"insertSpace",		"togglePlain",	"insertNBSpace",
-						  "",					"",					"",				""					}},
+						  "insertNBZWSpace",	"insertZWJoiner",					"",				""					}},
 	{EV_NVK_TAB,		{ "insertTab",			"",					"cycleWindows",	"cycleWindowsBck",
-						  "",					"",					"",				""					}},
+						  DO_NOT_USE,			"",					"",				""					}},
 	{EV_NVK_RETURN,		{ "insertParagraphBreak", "insertLineBreak", "insertPageBreak", "insertColumnBreak",
 						  "insertSectionBreak",	"",					"",				""					}},
 //	{EV_NVK_ESCAPE,		{ "",					"",					"",				"",
 //						  "",					"",					"",				""					}},
-	{EV_NVK_PAGEUP,		{ "scrollPageUp",		"extSelPageUp",		"warpInsPtPrevPage",	"",
+	{EV_NVK_PAGEUP,		{ "warpInsPtPrevScreen","extSelScreenUp",   "warpInsPtPrevPage",	"extSelPageUp",
 						  "",					"",					"warpInsPtBOP",	""					}},
-	{EV_NVK_PAGEDOWN,	{ "scrollPageDown",		"extSelPageDown",	"warpInsPtNextPage",	"",
+	{EV_NVK_PAGEDOWN,	{ "warpInsPtNextScreen","extSelScreenDown",	"warpInsPtNextPage",	"extSelPageDown",
 						  "",					"",					"warpInsPtEOP",	""					}},
 	{EV_NVK_END,		{ "warpInsPtEOL",		"extSelEOL",		"warpInsPtEOD",	"extSelEOD",
 						  "",					"",					"",				""					}},
@@ -227,7 +268,7 @@ ap_bs_NVK NVKTable[] =
 						  "",					"",					"",				""					}},
 //	{EV_NVK_F2,			{ "",					"",					"",				"",
 //						  "",					"",					"",				""					}},
-	{EV_NVK_F3,			{ "findAgain",			"",					"",				"",
+	{EV_NVK_F3,			{ "findAgain",			"rotateCase",		"",				"",
 						  "",					"",					"",				""					}},
 	{EV_NVK_F4,			{ "",					"",					"closeWindow",	"",
 						  "querySaveAndExit",	"",					"",				""					}},
@@ -283,7 +324,7 @@ DEFLEN(NVKTable);
 
 ap_bs_NVK_Prefix NVKTable_P[] =
 {
-//	{nvk,						{ none,					_S,					_C,				_S_C,		
+//	{nvk,						{ none,					_S,					_C,				_S_C,
 //  							  _A,					_A_S,				_A_C,			_A_C_S	}},
 
 	{EV_NVK_DEAD_GRAVE,			{ "deadgrave",			"deadgrave",		"",	"", "",	"",	"",	""	}},
@@ -322,98 +363,98 @@ ap_bs_Char CharTable[] =
 	{0x25, /* %      */ { "insertData",			"",					"",				""					}},
 	{0x26, /* &      */ { "insertData",			"",					"",				""					}},
 	{0x27, /* '      */ { "insertData",			"",					"",				""					}},
-	{0x28, /* (      */ { "insertData",			"",					"",				""					}},
-	{0x29, /* )      */ { "insertData",			"",					"",				""					}},
+	{0x28, /* (      */ { "insertOpeningParenthesis","",			"",				""					}},
+	{0x29, /* )      */ { "insertClosingParenthesis","",			"",				""					}},
 	{0x2a, /* *      */ { "insertData",			"viewPara",			"",				""					}},
 	{0x2b, /* +      */ { "insertData",			"",					"",				""					}},
 	{0x2c, /* ,      */ { "insertData",			"",					"",				""					}},
-	{0x2d, /* -      */ { "insertData",			"toggleSub",		"",				""					}},
-	{0x2e, /* .      */ { "insertData",			"",					"",				""					}},
+	{0x2d, /* -      */ { "insertData",			"toggleSub",		"zoomOut",		""					}},
+	{0x2e, /* .      */ { "insertData",			"pasteSpecial",		"",				""					}},
 	{0x2f, /* /      */ { "insertData",			"",					"",				""					}},
-	{0x30, /* 0      */ { "insertData",			"",					"",				""					}},
-	{0x31, /* 1      */ { "insertData",			"singleSpace",		"setStyleHeading1",				""					}},
-	{0x32, /* 2      */ { "insertData",			"doubleSpace",		"setStyleHeading2",				""					}},
-	{0x33, /* 3      */ { "insertData",			"",					"setStyleHeading3",				""					}},
-	{0x34, /* 4      */ { "insertData",			"",					"",				""					}},
-	{0x35, /* 5      */ { "insertData",			"middleSpace",		"",				""					}},
-	{0x36, /* 6      */ { "insertData",			"",					"",				""					}},
-	{0x37, /* 7      */ { "insertData",			"",					"",				""					}},
-	{0x38, /* 8      */ { "insertData",			"",					"",				""					}},
-	{0x39, /* 9      */ { "insertData",			"",					"",				""					}},
+	{0x30, /* 0      */ { "insertData",			"",					DO_NOT_USE,		""					}},
+	{0x31, /* 1      */ { "insertData",			"singleSpace",		DO_NOT_USE,		"setStyleHeading1"	}},
+	{0x32, /* 2      */ { "insertData",			"doubleSpace",		DO_NOT_USE,		"setStyleHeading2"	}},
+	{0x33, /* 3      */ { "insertData",			"",					DO_NOT_USE,		"setStyleHeading3"	}},
+	{0x34, /* 4      */ { "insertData",			"",					DO_NOT_USE,		""					}},
+	{0x35, /* 5      */ { "insertData",			"middleSpace",		DO_NOT_USE,		""					}},
+	{0x36, /* 6      */ { "insertData",			"",					DO_NOT_USE,		""					}},
+	{0x37, /* 7      */ { "insertData",			"",					DO_NOT_USE,		""					}},
+	{0x38, /* 8      */ { "insertData",			"",					DO_NOT_USE,		""					}},
+	{0x39, /* 9      */ { "insertData",			"",					DO_NOT_USE,		""					}},
 	{0x3a, /* :      */ { "insertData",			"",					"",				""					}},
 	{0x3b, /* ;      */ { "insertData",			"",					"",				""					}},
-	{0x3c, /* <      */ { "insertData",			"",					"",				""					}},
-	{0x3d, /* =      */ { "insertData",			"toggleSuper",		"",				""					}},
-	{0x3e, /* >      */ { "insertData",			"",					"",				""					}},
+	{0x3c, /* <      */ { "insertData",			"fontSizeIncrease",	"",				"insertRLM"			}},
+	{0x3d, /* =      */ { "insertData",			"toggleSuper",		"zoomIn",		""					}},
+	{0x3e, /* >      */ { "insertData",			"fontSizeDecrease",	"",				"insertLRM"			}},
 	{0x3f, /* ?      */ { "insertData",			"",					"",				""					}},
 	{0x40, /* @      */ { "insertData",			"",					"",				""					}},
-	{0x41, /* A      */ { "insertData",			"selectAll",		"",				""					}},
-	{0x42, /* B      */ { "insertData",			"toggleBold",		"",				""					}},
-	{0x43, /* C      */ { "insertData",			"copy",				"",				""					}},
-	{0x44, /* D      */ { "insertData",			"dlgFont",			"",				""					}},
-	{0x45, /* E      */ { "insertData",			"alignCenter",		"",				""					}},
-	{0x46, /* F      */ { "insertData",			"find",				"",				""					}},
-	{0x47, /* G      */ { "insertData",			"go",				"",				""					}},
-	{0x48, /* H      */ { "insertData",			"replace",			"",				""					}},
-	{0x49, /* I      */ { "insertData",			"toggleItalic",		"",				""					}},
-	{0x4a, /* J      */ { "insertData",			"alignJustify",		"",				""					}},
-	{0x4b, /* K      */ { "insertData",			"toggleStrike",		"",				""					}},
-	{0x4c, /* L      */ { "insertData",			"alignLeft",		"",				""					}},
-	{0x4d, /* M      */ { "insertData",			"insSymbol",		"",				""					}},
-	{0x4e, /* N      */ { "insertData",			"toolbarNew",		"",				""					}},
-	{0x4f, /* O      */ { "insertData",			"fileOpen",			"",				""					}},
-	{0x50, /* P      */ { "insertData",			"print",			"",				""					}},
-	{0x51, /* Q      */ { "insertData",			"querySaveAndExit",	"",				""					}},
-	{0x52, /* R      */ { "insertData",			"alignRight",		"",				""					}},
-	{0x53, /* S      */ { "insertData",			"fileSave",			"",				""					}},
-	{0x54, /* T      */ { "insertData",			"toggleOline",		"",				""					}},
-	{0x55, /* U      */ { "insertData",			"toggleUline",		"",				""					}},
-	{0x56, /* V      */ { "insertData",			"paste",			"",				""					}},
-	{0x57, /* W      */ { "insertData",			"closeWindow",		"",				""					}},
-	{0x58, /* X      */ { "insertData",			"cut",				"",				""					}},
-	{0x59, /* Y      */ { "insertData",			"redo",				"",				""					}},
-	{0x5a, /* Z      */ { "insertData",			"undo",				"",				""					}},
-	{0x5b, /* [      */ { "insertData",			"editHeader",					"",				""					}},
-	{0x5c, /* \      */ { "insertData",			"removeHeader",					"",				""					}},
-	{0x5d, /* ]      */ { "insertData",			"editFooter",					"",				""					}},
-	{0x5e, /* ^      */ { "insertData",			"removeFooter",					"",				""					}},
+	{0x41, /* A      */ { "insertData",			"selectAll",		DO_NOT_USE,		""					}},
+	{0x42, /* B      */ { "insertData",			"toggleBold",		DO_NOT_USE,		""					}},
+	{0x43, /* C      */ { "insertData",			"copy",				DO_NOT_USE,		""					}},
+	{0x44, /* D      */ { "insertData",			"dlgFont",			DO_NOT_USE,		""					}},
+	{0x45, /* E      */ { "insertData",			"alignCenter",		DO_NOT_USE,		""					}},
+	{0x46, /* F      */ { "insertData",			"find",				DO_NOT_USE,		""					}},
+	{0x47, /* G      */ { "insertData",			"go",				DO_NOT_USE,		""					}},
+	{0x48, /* H      */ { "insertData",			"replace",			DO_NOT_USE,		""					}},
+	{0x49, /* I      */ { "insertData",			"toggleItalic",		DO_NOT_USE,		""					}},
+	{0x4a, /* J      */ { "insertData",			"alignJustify",		DO_NOT_USE,		""					}},
+	{0x4b, /* K      */ { "insertData",			"toggleStrike",		DO_NOT_USE,		""					}},
+	{0x4c, /* L      */ { "insertData",			"alignLeft",		DO_NOT_USE,		""					}},
+	{0x4d, /* M      */ { "insertData",			"insSymbol",		DO_NOT_USE,		""					}},
+	{0x4e, /* N      */ { "insertData",			"toolbarNew",		DO_NOT_USE,		""					}},
+	{0x4f, /* O      */ { "insertData",			"fileOpen",			DO_NOT_USE,		""					}},
+	{0x50, /* P      */ { "insertData",			"print",			DO_NOT_USE,		""					}},
+	{0x51, /* Q      */ { "insertData",			"querySaveAndExit",	DO_NOT_USE,		""					}},
+	{0x52, /* R      */ { "insertData",			"alignRight",		DO_NOT_USE,		""					}},
+	{0x53, /* S      */ { "insertData",			"fileSave",			DO_NOT_USE,		""					}},
+	{0x54, /* T      */ { "insertData",			"toggleOline",		DO_NOT_USE,		""					}},
+	{0x55, /* U      */ { "insertData",			"toggleUline",		DO_NOT_USE,		""					}},
+	{0x56, /* V      */ { "insertData",			"paste",			DO_NOT_USE,		""					}},
+	{0x57, /* W      */ { "insertData",			"closeWindow",		DO_NOT_USE,		""					}},
+	{0x58, /* X      */ { "insertData",			"cut",				DO_NOT_USE,		""					}},
+	{0x59, /* Y      */ { "insertData",			"redo",				DO_NOT_USE,		""					}},
+	{0x5a, /* Z      */ { "insertData",			"undo",				DO_NOT_USE,		""					}},
+	{0x5b, /* [      */ { "insertOpeningParenthesis","editHeader",	"",				""					}},
+	{0x5c, /* \      */ { "insertData",			"removeHeader",		"",				""					}},
+	{0x5d, /* ]      */ { "insertClosingParenthesis","editFooter",	"",				""					}},
+	{0x5e, /* ^      */ { "insertData",			"removeFooter",		"",				""					}},
 	{0x5f, /* -      */ { "insertData",			"",					"",				""					}},
 	{0x60, /* `      */ { "insertData",			"",					"",				""					}},
-	{0x61, /* a      */ { "insertData",			"selectAll",		"",				""					}},
-	{0x62, /* b      */ { "insertData",			"toggleBold",		"",				""					}},
-	{0x63, /* c      */ { "insertData",			"copy",				"",				""					}},
-	{0x64, /* d      */ { "insertData",			"dlgFont",			"",				""					}},
-	{0x65, /* e      */ { "insertData",			"alignCenter",		"",				""					}},
-	{0x66, /* f      */ { "insertData",			"find",				"",				""					}},
-	{0x67, /* g      */ { "insertData",			"go",				"",				""					}},
-	{0x68, /* h      */ { "insertData",			"replace",			"",				""					}},
-	{0x69, /* i      */ { "insertData",			"toggleItalic",		"",				""					}},
-	{0x6a, /* j      */ { "insertData",			"alignJustify",		"",				""					}},
-	{0x6b, /* k      */ { "insertData",			"toggleStrike",		"",				""					}},
-	{0x6c, /* l      */ { "insertData",			"alignLeft",		"",				""					}},
-	{0x6d, /* m      */ { "insertData",			"insSymbol",		"",				""					}},
-	{0x6e, /* n      */ { "insertData",			"toolbarNew",		"",				""					}},
-	{0x6f, /* o      */ { "insertData",			"fileOpen",			"",				""					}},
-	{0x70, /* p      */ { "insertData",			"print",			"",				""					}},
-	{0x71, /* q      */ { "insertData",			"querySaveAndExit",	"",				""					}},
-	{0x72, /* r      */ { "insertData",			"alignRight",		"",				""					}},
-	{0x73, /* s      */ { "insertData",			"fileSave",			"",				""					}},
-	{0x74, /* t      */ { "insertData",			"toggleOline",		"",				""					}},
-	{0x75, /* u      */ { "insertData",			"toggleUline",		"",				""					}},
-	{0x76, /* v      */ { "insertData",			"paste",			"",				""					}},
-	{0x77, /* w      */ { "insertData",			"closeWindow",		"",				""					}},
-	{0x78, /* x      */ { "insertData",			"cut",				"",				""					}},
-	{0x79, /* y      */ { "insertData",			"redo",				"",				""					}},
-	{0x7a, /* z      */ { "insertData",			"undo",				"",				""					}},
-	{0x7b, /* {      */ { "insertData",			"",					"",				""					}},
+	{0x61, /* a      */ { "insertData",			"selectAll",		DO_NOT_USE,		""					}},
+	{0x62, /* b      */ { "insertData",			"toggleBold",		DO_NOT_USE,		""					}},
+	{0x63, /* c      */ { "insertData",			"copy",				DO_NOT_USE,		""					}},
+	{0x64, /* d      */ { "insertData",			"dlgFont",			DO_NOT_USE,		""					}},
+	{0x65, /* e      */ { "insertData",			"alignCenter",		DO_NOT_USE,		""					}},
+	{0x66, /* f      */ { "insertData",			"find",				DO_NOT_USE,		""					}},
+	{0x67, /* g      */ { "insertData",			"go",				DO_NOT_USE,		""					}},
+	{0x68, /* h      */ { "insertData",			"replace",			DO_NOT_USE,		""					}},
+	{0x69, /* i      */ { "insertData",			"toggleItalic",		DO_NOT_USE,		""					}},
+	{0x6a, /* j      */ { "insertData",			"alignJustify",		DO_NOT_USE,		""					}},
+	{0x6b, /* k      */ { "insertData",			"toggleStrike",		DO_NOT_USE,		""					}},
+	{0x6c, /* l      */ { "insertData",			"alignLeft",		DO_NOT_USE,		""					}},
+	{0x6d, /* m      */ { "insertData",			"insSymbol",		DO_NOT_USE,		""					}},
+	{0x6e, /* n      */ { "insertData",			"toolbarNew",		DO_NOT_USE,		""					}},
+	{0x6f, /* o      */ { "insertData",			"fileOpen",			DO_NOT_USE,		""					}},
+	{0x70, /* p      */ { "insertData",			"print",			DO_NOT_USE,		""					}},
+	{0x71, /* q      */ { "insertData",			"querySaveAndExit",	DO_NOT_USE,		""					}},
+	{0x72, /* r      */ { "insertData",			"alignRight",		DO_NOT_USE,		""					}},
+	{0x73, /* s      */ { "insertData",			"fileSave",			DO_NOT_USE,		""					}},
+	{0x74, /* t      */ { "insertData",			"toggleOline",		DO_NOT_USE,		""					}},
+	{0x75, /* u      */ { "insertData",			"toggleUline",		DO_NOT_USE,		""					}},
+	{0x76, /* v      */ { "insertData",			"paste",			DO_NOT_USE,		""					}},
+	{0x77, /* w      */ { "insertData",			"closeWindow",		DO_NOT_USE,		""					}},
+	{0x78, /* x      */ { "insertData",			"cut",				DO_NOT_USE,		""					}},
+	{0x79, /* y      */ { "insertData",			"redo",				DO_NOT_USE,		""					}},
+	{0x7a, /* z      */ { "insertData",			"undo",				DO_NOT_USE,		""					}},
+	{0x7b, /* {      */ { "insertOpeningParenthesis","",			"",				""					}},
 	{0x7c, /* |      */ { "insertData",			"",					"",				""					}},
-	{0x7d, /* }      */ { "insertData",			"",					"",				""					}},
+	{0x7d, /* }      */ { "insertClosingParenthesis","",			"",				""					}},
 	{0x7e, /* ~      */ { "insertData",			"",					"",				""					}},
 
 //	Here are bindings for the portion of Latin-1 with the high-bit set.
 //	I'm taking the list from /usr/include/X11/keysymdef.h, so I think
 //	that the set is reasonably complete.
-//	
+//
 //	I don't know how to test these on my en-US system with my en-US keyboard,
 //	but I think they will work.
 //
