@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -40,6 +40,11 @@
 #include "pd_Document.h"
 #include "ut_Script.h"
 #include "spell_manager.h"
+#include "ie_mailmerge.h"
+
+#ifdef _WIN32
+#include "ap_Win32App.h" 
+#endif
 
 #define ABIWORD_VIEW  	FV_View * pView = static_cast<FV_View *>(pAV_View)
 
@@ -60,86 +65,86 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Autotext)
 
 	switch (id)
 	  {
-	  case AP_MENU_ID_AUTOTEXT_ATTN_1: 
+	  case AP_MENU_ID_AUTOTEXT_ATTN_1:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_ATTN_1); break;
-	  case AP_MENU_ID_AUTOTEXT_ATTN_2: 
+	  case AP_MENU_ID_AUTOTEXT_ATTN_2:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_ATTN_2); break;
 
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_1: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_1:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_1); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_2: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_2:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_2); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_3: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_3:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_3); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_4: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_4:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_4); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_5: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_5:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_5); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_6: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_6:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_6); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_7: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_7:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_7); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_8: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_8:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_8); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_9: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_9:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_9); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_10: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_10:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_10); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_11: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_11:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_11); break;
-	  case AP_MENU_ID_AUTOTEXT_CLOSING_12: 
+	  case AP_MENU_ID_AUTOTEXT_CLOSING_12:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_CLOSING_12); break;
 
-	  case AP_MENU_ID_AUTOTEXT_MAIL_1: 
+	  case AP_MENU_ID_AUTOTEXT_MAIL_1:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_MAIL_1); break;
-	  case AP_MENU_ID_AUTOTEXT_MAIL_2: 
+	  case AP_MENU_ID_AUTOTEXT_MAIL_2:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_MAIL_2); break;
-	  case AP_MENU_ID_AUTOTEXT_MAIL_3: 
+	  case AP_MENU_ID_AUTOTEXT_MAIL_3:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_MAIL_3); break;
-	  case AP_MENU_ID_AUTOTEXT_MAIL_4: 
+	  case AP_MENU_ID_AUTOTEXT_MAIL_4:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_MAIL_4); break;
-	  case AP_MENU_ID_AUTOTEXT_MAIL_5: 
+	  case AP_MENU_ID_AUTOTEXT_MAIL_5:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_MAIL_5); break;
-	  case AP_MENU_ID_AUTOTEXT_MAIL_6: 
+	  case AP_MENU_ID_AUTOTEXT_MAIL_6:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_MAIL_6); break;
-	  case AP_MENU_ID_AUTOTEXT_MAIL_7: 
+	  case AP_MENU_ID_AUTOTEXT_MAIL_7:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_MAIL_7); break;
-	  case AP_MENU_ID_AUTOTEXT_MAIL_8: 
+	  case AP_MENU_ID_AUTOTEXT_MAIL_8:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_MAIL_8); break;
 
-	  case AP_MENU_ID_AUTOTEXT_REFERENCE_1: 
+	  case AP_MENU_ID_AUTOTEXT_REFERENCE_1:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_REFERENCE_1); break;
-	  case AP_MENU_ID_AUTOTEXT_REFERENCE_2: 
+	  case AP_MENU_ID_AUTOTEXT_REFERENCE_2:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_REFERENCE_2); break;
-	  case AP_MENU_ID_AUTOTEXT_REFERENCE_3: 
+	  case AP_MENU_ID_AUTOTEXT_REFERENCE_3:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_REFERENCE_3); break;
 
-	  case AP_MENU_ID_AUTOTEXT_SALUTATION_1: 
+	  case AP_MENU_ID_AUTOTEXT_SALUTATION_1:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_SALUTATION_1); break;
-	  case AP_MENU_ID_AUTOTEXT_SALUTATION_2: 
+	  case AP_MENU_ID_AUTOTEXT_SALUTATION_2:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_SALUTATION_2); break;
-	  case AP_MENU_ID_AUTOTEXT_SALUTATION_3: 
+	  case AP_MENU_ID_AUTOTEXT_SALUTATION_3:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_SALUTATION_3); break;
-	  case AP_MENU_ID_AUTOTEXT_SALUTATION_4: 
+	  case AP_MENU_ID_AUTOTEXT_SALUTATION_4:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_SALUTATION_4); break;
 
-	  case AP_MENU_ID_AUTOTEXT_SUBJECT_1: 
+	  case AP_MENU_ID_AUTOTEXT_SUBJECT_1:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_SUBJECT_1); break;
 
-	  case AP_MENU_ID_AUTOTEXT_EMAIL_1: 
+	  case AP_MENU_ID_AUTOTEXT_EMAIL_1:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_EMAIL_1); break;
-	  case AP_MENU_ID_AUTOTEXT_EMAIL_2: 
+	  case AP_MENU_ID_AUTOTEXT_EMAIL_2:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_EMAIL_2); break;
-	  case AP_MENU_ID_AUTOTEXT_EMAIL_3: 
+	  case AP_MENU_ID_AUTOTEXT_EMAIL_3:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_EMAIL_3); break;
-	  case AP_MENU_ID_AUTOTEXT_EMAIL_4: 
+	  case AP_MENU_ID_AUTOTEXT_EMAIL_4:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_EMAIL_4); break;
-	  case AP_MENU_ID_AUTOTEXT_EMAIL_5: 
+	  case AP_MENU_ID_AUTOTEXT_EMAIL_5:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_EMAIL_5); break;
-	  case AP_MENU_ID_AUTOTEXT_EMAIL_6: 
+	  case AP_MENU_ID_AUTOTEXT_EMAIL_6:
 	    c = pss->getValue(AP_STRING_ID_AUTOTEXT_EMAIL_6); break;
-	    
-	  default: 
+
+	  default:
 	    c = "No clue"; break;
 	  }
 
@@ -151,7 +156,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Recent)
 	// Compute the menu label for _recent_1 thru _recent_9 on the menu.
 	// We return a pointer to a static string (which will be overwritten
 	// on the next call).
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
@@ -159,7 +164,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Recent)
 
 	UT_ASSERT(id >= AP_MENU_ID_FILE_RECENT_1);
 	UT_ASSERT(id <= AP_MENU_ID_FILE_RECENT_9);
-	
+
 	UT_uint32 ndx = (id - AP_MENU_ID_FILE_RECENT_1 + 1);
 
 	XAP_Prefs * pPrefs = pApp->getPrefs();
@@ -172,10 +177,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Recent)
 
 		const char * szRecent = pPrefs->getRecent(ndx);
 
-		sprintf(buf,szFormat,szRecent);	
+		sprintf(buf,szFormat,szRecent);
 		return buf;
 	}
-	
+
 	// for the other slots, return a null string to tell
 	// the menu code to remove this item from the menu.
 
@@ -185,120 +190,120 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Recent)
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_About)
 {
 	// Compute the menu label for the _help_about item.
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
 	UT_ASSERT(pLabel);
 
 	UT_ASSERT(id == AP_MENU_ID_HELP_ABOUT);
-	
+
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
 
 	const char * szAppName = pApp->getApplicationName();
 
-	sprintf(buf,szFormat,szAppName);	
+	sprintf(buf,szFormat,szAppName);
 	return buf;
 }
 
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Contents)
 {
 	// Compute the menu label for the _help_contents item.
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
 	UT_ASSERT(pLabel);
 
 	UT_ASSERT(id == AP_MENU_ID_HELP_CONTENTS);
-	
+
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
 
 	const char * szAppName = pApp->getApplicationName();
 
-	sprintf(buf,szFormat,szAppName);	
+	sprintf(buf,szFormat,szAppName);
 	return buf;
 }
 
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Index)
 {
 	// Compute the menu label for the _help_index item.
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
 	UT_ASSERT(pLabel);
 
 	UT_ASSERT(id == AP_MENU_ID_HELP_INDEX);
-	
+
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
 
 	const char * szAppName = pApp->getApplicationName();
 
-	sprintf(buf,szFormat,szAppName);	
+	sprintf(buf,szFormat,szAppName);
 	return buf;
 }
 
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Search)
 {
 	// Compute the menu label for the _help_search item.
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
 	UT_ASSERT(pLabel);
 
 	UT_ASSERT(id == AP_MENU_ID_HELP_SEARCH);
-	
+
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
 
 	const char * szAppName = pApp->getApplicationName();
 
-	sprintf(buf,szFormat,szAppName);	
+	sprintf(buf,szFormat,szAppName);
 	return buf;
 }
 
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_AboutOS)
 {
 	// Compute the menu label for the about OS help item.
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
 	UT_ASSERT(pLabel);
 
 	UT_ASSERT(id == AP_MENU_ID_HELP_ABOUTOS);
-	
+
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
 
 	const char * szAppName = pApp->getApplicationName();
 
-	sprintf(buf,szFormat,szAppName);	
+	sprintf(buf,szFormat,szAppName);
 	return buf;
 }
 
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Checkver)
 {
 	// Compute the menu label for the about the check version item.
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
 	UT_ASSERT(pLabel);
 
 	UT_ASSERT(id == AP_MENU_ID_HELP_CHECKVER);
-	
+
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
 
 	const char * szAppName = pApp->getApplicationName();
 
-	sprintf(buf,szFormat,szAppName);	
+	sprintf(buf,szFormat,szAppName);
 	return buf;
 }
 /*****************************************************************/
@@ -310,12 +315,12 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Window)
 
 	UT_ASSERT(id >= AP_MENU_ID_WINDOW_1);
 	UT_ASSERT(id <= AP_MENU_ID_WINDOW_9);
-	
+
 	UT_uint32 ndx = (id - AP_MENU_ID_WINDOW_1);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
-	
-	XAP_Frame * pFrame = (XAP_Frame *) pAV_View->getParentData();
+
+	XAP_Frame * pFrame = static_cast<XAP_Frame *>(pAV_View->getParentData());
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
@@ -331,7 +336,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Window)
 	// Compute the menu label for _window_1 thru _window_9 on the menu.
 	// We return a pointer to a static string (which will be overwritten
 	// on the next call).
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
@@ -340,7 +345,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Window)
 
 	UT_ASSERT(id >= AP_MENU_ID_WINDOW_1);
 	UT_ASSERT(id <= AP_MENU_ID_WINDOW_9);
-	
+
 	UT_uint32 ndx = (id - AP_MENU_ID_WINDOW_1);
 
 	// use the applications window list and compute a menu label
@@ -357,10 +362,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Window)
 
 		const char * szTitle = pFrame->getTitle(128 - strlen(szFormat));
 
-		sprintf(buf,szFormat,szTitle);	
+		sprintf(buf,szFormat,szTitle);
 		return buf;
 	}
-	
+
 	// for the other slots, return a null string to tell
 	// the menu code to remove this item from the menu.
 
@@ -370,13 +375,13 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Window)
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_WindowMore)
 {
 	// Compute the menu label for the _window_more ("More Windows...") item.
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
 	UT_ASSERT(pLabel);
 	UT_ASSERT(id == AP_MENU_ID_WINDOW_MORE);
-	
+
 	// if we have more than 9 windows in our window list,
 	// we return the static menu label.  if not, we return
 	// null string to tell the menu code to remove this
@@ -398,7 +403,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Spelling)
   UT_ASSERT( pPrefs );
 
   bool b = true ;
-  pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_AutoSpellCheck,&b) ;
+  pPrefs->getPrefsValueBool(static_cast<XML_Char*>(AP_PREF_KEY_AutoSpellCheck),&b) ;
 
   // if there are no loaded dictionaries and we are spell checking
   // as we type
@@ -445,7 +450,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Suggest)
 
 	UT_ASSERT(id >= AP_MENU_ID_SPELL_SUGGEST_1);
 	UT_ASSERT(id <= AP_MENU_ID_SPELL_SUGGEST_9);
-	
+
 	UT_uint32 ndx = (id - AP_MENU_ID_SPELL_SUGGEST_1 + 1);
 
 	EV_Menu_ItemState s = EV_MIS_Gray;
@@ -467,7 +472,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Suggest)
 	// Compute the menu label for _suggest_1 thru _suggest_9 on the menu.
 	// We return a pointer to a static string (which will be overwritten
 	// on the next call).
-	
+
 	UT_ASSERT(pFrame);
 	XAP_App * pApp = pFrame->getApp();
 	UT_ASSERT(pApp);
@@ -478,7 +483,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Suggest)
 
 	UT_ASSERT(id >= AP_MENU_ID_SPELL_SUGGEST_1);
 	UT_ASSERT(id <= AP_MENU_ID_SPELL_SUGGEST_9);
-	
+
 	UT_uint32 ndx = (id - AP_MENU_ID_SPELL_SUGGEST_1 + 1);
 
 	const char * c = NULL;
@@ -488,13 +493,23 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Suggest)
 
 	p = pView->getContextSuggest(ndx);
 	if (p && *p)
-		len = UT_UCS_strlen(p);
+		len = UT_UCS4_strlen(p);
 
 	if (len)
 	{
 		// this is a suggestion
-		UT_UCS_strcpy_to_char(cBuf, p);
+		char *outbuf = cBuf;
+		int i;
+		for (i = 0; i < static_cast<int>(len); i++) {
+			outbuf += unichar_to_utf8(p[i], reinterpret_cast<unsigned char *>(outbuf));
+		}
+		*outbuf = 0;
 		c = cBuf;
+
+		#ifdef _WIN32	// UTF-8 to ANSI conversion for win32 build
+		UT_String 	sAnsi =	AP_Win32App::s_fromUTF8ToAnsi(cBuf);
+		strcpy (cBuf, sAnsi.c_str());
+		#endif
 	}
 	else if (ndx == 1)
 	{
@@ -511,7 +526,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Suggest)
 		static char buf[128];	// BUGBUG: possible buffer overflow
 
 		sprintf(buf,szFormat,c);
-		
+
 		return buf;
 	}
 
@@ -539,8 +554,11 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 
 	switch(id)
 	{
-	case AP_MENU_ID_FILE_REVERT:
 	case AP_MENU_ID_FILE_SAVE:
+	  if (!pView->getDocument()->isDirty() || !pView->canDo(true) || pView->getDocument()->getFileName() == NULL)
+	    s = EV_MIS_Gray;
+	  break;
+	case AP_MENU_ID_FILE_REVERT:
 	  if (!pView->getDocument()->isDirty() || !pView->canDo(true))
 	    s = EV_MIS_Gray;
 	  break;
@@ -555,13 +573,23 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 		break;
 
 	case AP_MENU_ID_EDIT_EDITHEADER:
-//	  if (pFrameData->m_pViewMode == VIEW_NORMAL)
-//	    s = EV_MIS_Gray;
+		if (!pView->isHeaderOnPage())
+			s = EV_MIS_Gray;
 	  break;
 
 	case AP_MENU_ID_EDIT_EDITFOOTER:
-//	  if (pFrameData->m_pViewMode == VIEW_NORMAL)
-//	    s = EV_MIS_Gray;
+		if (!pView->isFooterOnPage())
+			s = EV_MIS_Gray;
+	  break;
+
+	case AP_MENU_ID_INSERT_INSERTHEADER:
+		if (pView->isHeaderOnPage())
+			s = EV_MIS_Gray;
+	  break;
+
+	case AP_MENU_ID_INSERT_INSERTFOOTER:
+		if (pView->isFooterOnPage())
+			s = EV_MIS_Gray;
 	  break;
 
 	case AP_MENU_ID_EDIT_REMOVEHEADER:
@@ -574,6 +602,21 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 			s = EV_MIS_Gray;
 		break;
 
+	case AP_MENU_ID_TABLE_INSERT:
+		if (pView->isHdrFtrEdit())
+			s = EV_MIS_Gray;
+		break;
+
+	case AP_MENU_ID_TABLE_INSERT_TABLE:
+		if (pView->isHdrFtrEdit())
+			s = EV_MIS_Gray;
+		break;
+
+	case AP_MENU_ID_TABLE_INSERTTABLE:
+		if (pView->isHdrFtrEdit())
+			s = EV_MIS_Gray;
+		break;
+
 	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		break;
@@ -598,7 +641,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_ScriptsActive)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Selection)
 {
         ABIWORD_VIEW;
-	XAP_App * pApp = pView->getApp();
+	XAP_App * pApp = XAP_App::getApp();
 	UT_ASSERT(pApp);
 
 	XAP_Prefs * pPrefs = pApp->getPrefs();
@@ -634,7 +677,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Clipboard)
 	{
 	case AP_MENU_ID_EDIT_PASTE_SPECIAL:
 	case AP_MENU_ID_EDIT_PASTE:
-		s = ( (pView->getApp()->canPasteFromClipboard()) ? EV_MIS_ZERO : EV_MIS_Gray );
+		s = ((XAP_App::getApp()->canPasteFromClipboard()) ? EV_MIS_ZERO : EV_MIS_Gray );
 		break;
 
 	default:
@@ -650,7 +693,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Prefs)
         ABIWORD_VIEW;
 	UT_ASSERT(pView);
 
-	XAP_App *pApp = pView->getApp();
+	XAP_App *pApp = XAP_App::getApp();
 	UT_ASSERT(pApp);
 
 	XAP_Prefs * pPrefs = pApp->getPrefs();
@@ -663,7 +706,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Prefs)
 	switch (id)
 	  {
 	  case AP_MENU_ID_TOOLS_AUTOSPELL:
-	    pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_AutoSpellCheck, &b);
+	    pPrefs->getPrefsValueBool(static_cast<XML_Char*>(AP_PREF_KEY_AutoSpellCheck), &b);
 	    s = (b ? EV_MIS_Toggled : EV_MIS_ZERO);
 	    break;
 
@@ -686,7 +729,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 	const XML_Char * prop = NULL;
 	const XML_Char * val  = NULL;
 
-	if(pView->getDocument()->areStylesLocked()) {
+	if(pView->getDocument()->areStylesLocked() && (AP_MENU_ID_FMT_SUPERSCRIPT != id || AP_MENU_ID_FMT_SUBSCRIPT != id)) {
           return EV_MIS_Gray;
 	}
 
@@ -740,7 +783,17 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 		prop = "text-position";
 		val  = "subscript";
 		break;
-	
+
+	case AP_MENU_ID_FMT_DIRECTION_DO_RTL:
+		prop = "dir-override";
+		val  = "rtl";
+		break;
+		
+	case AP_MENU_ID_FMT_DIRECTION_DO_LTR:
+		prop = "dir-override";
+		val  = "ltr";
+		break;
+
 	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		break;
@@ -759,7 +812,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 		if (sz)
 		{
 			if (bMultiple)
-			{	
+			{
 				// some properties have multiple values
 				if (strstr(sz, val))
 					s = EV_MIS_Toggled;
@@ -770,7 +823,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 					s = EV_MIS_Toggled;
 			}
 		}
-		
+
 		free(props_in);
 	}
 
@@ -810,6 +863,11 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BlockFmt)
 		val  = "justify";
 		break;
 
+	case AP_MENU_ID_FMT_DIRECTION_DD_RTL:
+		prop = "dom-dir";
+		val  = "rtl";
+		break;
+
 	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		break;
@@ -827,7 +885,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BlockFmt)
 		sz = UT_getAttribute(prop, props_in);
 		if (sz && (0 == UT_strcmp(sz, val)))
 			s = EV_MIS_Toggled;
-		
+
 		free(props_in);
 	}
 
@@ -839,19 +897,21 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	ABIWORD_VIEW;
 	UT_ASSERT(pView);
 
-	XAP_Frame * pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
+	XAP_Frame *pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
 	UT_ASSERT(pFrame);
 
 	AP_FrameData *pFrameData = static_cast<AP_FrameData *> (pFrame->getFrameData());
 	UT_ASSERT(pFrameData);
 
-	EV_Menu_ItemState s = EV_MIS_ZERO;
+	XAP_App *pApp = pFrame->getApp();
+	UT_ASSERT(pApp);
 
+	EV_Menu_ItemState s = EV_MIS_ZERO;
 
 	switch(id)
 	{
 	case AP_MENU_ID_VIEW_RULER:
-		if ( pFrameData->m_bShowRuler && !pFrameData->m_bIsFullScreen) 
+		if ( pFrameData->m_bShowRuler && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
 		else
 			s = EV_MIS_ZERO;
@@ -862,30 +922,48 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 		else
 		  s = EV_MIS_ZERO;
 		break;
-//	case AP_MENU_ID_VIEW_HEADFOOT:
-//		// TODO: implement view methods to check, toggle state
-//		s = EV_MIS_Gray;
-//		break;
+	case AP_MENU_ID_VIEW_LOCKSTYLES:
+		if ( pView->getDocument()->areStylesLocked() )
+			s = EV_MIS_ZERO;
+		else
+			s = EV_MIS_Toggled;
+		break;
 	case AP_MENU_ID_VIEW_TB_STD:
-		if ( pFrameData->m_bShowBar[0] && !pFrameData->m_bIsFullScreen) 
+		if ( pFrameData->m_bShowBar[0] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_FORMAT:
-		if ( pFrameData->m_bShowBar[1] && !pFrameData->m_bIsFullScreen) 
+		if ( pFrameData->m_bShowBar[1] && !pFrameData->m_bIsFullScreen)
+			s = EV_MIS_Toggled;
+		else
+			s = EV_MIS_ZERO;
+		break;
+	case AP_MENU_ID_VIEW_TB_TABLE:
+		if ( pFrameData->m_bShowBar[2] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_EXTRA:
-		if ( pFrameData->m_bShowBar[2] && !pFrameData->m_bIsFullScreen) 
+		if ( pFrameData->m_bShowBar[3] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
 		else
 			s = EV_MIS_ZERO;
 		break;
+	case AP_MENU_ID_VIEW_LOCK_TB_LAYOUT:
+		if ( !pApp->areToolbarsCustomizable() )
+			s = EV_MIS_Toggled;
+		else
+			s = EV_MIS_ZERO;
+		break;
+	case AP_MENU_ID_VIEW_DEFAULT_TB_LAYOUT:
+		if ( !pApp->areToolbarsCustomizable() || !pApp->areToolbarsCustomized() )
+			s = EV_MIS_Gray;
+		break;
 	case AP_MENU_ID_VIEW_STATUSBAR:
-              if ( pFrameData->m_bShowStatusBar && 
+              if ( pFrameData->m_bShowStatusBar &&
 				   !pFrameData->m_bIsFullScreen )
 				  s = EV_MIS_Toggled;
 			  else
@@ -900,21 +978,21 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 		break;
 
 	case AP_MENU_ID_VIEW_NORMAL:
-	  if ( pFrameData->m_pViewMode == VIEW_NORMAL) 
+	  if ( pFrameData->m_pViewMode == VIEW_NORMAL)
 	    s = EV_MIS_Toggled;
 	  else
 	    s = EV_MIS_ZERO;
 	  break;
 
-	case AP_MENU_ID_VIEW_WEB: 
-	  if ( pFrameData->m_pViewMode == VIEW_WEB) 
+	case AP_MENU_ID_VIEW_WEB:
+	  if ( pFrameData->m_pViewMode == VIEW_WEB)
 	    s = EV_MIS_Toggled;
 	  else
 	    s = EV_MIS_ZERO;
 	  break;
 
-	case AP_MENU_ID_VIEW_PRINT: 
-	  if ( pFrameData->m_pViewMode == VIEW_PRINT) 
+	case AP_MENU_ID_VIEW_PRINT:
+	  if ( pFrameData->m_pViewMode == VIEW_PRINT)
 	    s = EV_MIS_Toggled;
 	  else
 	    s = EV_MIS_ZERO;
@@ -939,3 +1017,154 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_StylesLocked)
 
         return EV_MIS_ZERO;
 }
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions)
+{
+	ABIWORD_VIEW;
+	UT_ASSERT(pView);
+
+        if(pView->isMarkRevisions()) {
+            return EV_MIS_Toggled;
+        }
+
+        return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresent)
+{
+	ABIWORD_VIEW;
+	UT_ASSERT(pView);
+
+	if(pView->isMarkRevisions())
+		return EV_MIS_Gray;
+    else if(pView->getInsertionPointContext(NULL,NULL) != EV_EMC_REVISION)
+        return EV_MIS_Gray;
+
+	return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresentContext)
+{
+	ABIWORD_VIEW;
+	UT_ASSERT(pView);
+
+	if(pView->isMarkRevisions())
+		return EV_MIS_Gray;
+
+    return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InTable)
+{
+	ABIWORD_VIEW;
+	UT_ASSERT(pView);
+
+	if(pView->isInTable())
+		return EV_MIS_ZERO;
+
+    return EV_MIS_Gray;
+}
+
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InFootnote)
+{
+	ABIWORD_VIEW;
+	UT_ASSERT(pView);
+
+	if(!pView->isInFootnote())
+	{
+		return EV_MIS_ZERO;
+	}
+	else
+	{
+		return EV_MIS_Gray;
+	}
+}
+
+// HACK TO ALWAYS DISABLE A MENU ITEM... DELETE ME
+Defun_EV_GetMenuItemState_Fn(ap_GetState_AlwaysDisabled)
+{
+    return EV_MIS_Gray;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_Recent)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail(pView, EV_MIS_ZERO);
+
+	XAP_Prefs *pPrefs = XAP_App::getApp()->getPrefs();
+	UT_return_val_if_fail(pPrefs, EV_MIS_ZERO);
+
+	if(pPrefs->getRecentCount() > 0)
+		return EV_MIS_ZERO;
+	
+	return EV_MIS_Gray;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_Zoom)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail(pView, EV_MIS_ZERO);
+
+	XAP_Frame * pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
+	UT_ASSERT(pFrame);
+
+	EV_Menu_ItemState s = EV_MIS_ZERO;
+
+	switch(id)
+	{
+	case AP_MENU_ID_VIEW_ZOOM_200:
+		if (pFrame->getZoomPercentage() == 200)
+			s = EV_MIS_Toggled;
+		break;
+	case AP_MENU_ID_VIEW_ZOOM_100:
+		if (pFrame->getZoomPercentage() == 100)
+			s = EV_MIS_Toggled;
+		break;
+	case AP_MENU_ID_VIEW_ZOOM_75:
+		if (pFrame->getZoomPercentage() == 75)
+			s = EV_MIS_Toggled;
+		break;
+	case AP_MENU_ID_VIEW_ZOOM_50:
+		if (pFrame->getZoomPercentage() == 50)
+			s = EV_MIS_Toggled;
+		break;
+	case AP_MENU_ID_VIEW_ZOOM_WHOLE:
+		xxx_UT_DEBUGMSG(("Whole: %d %d\n", pFrame->getZoomPercentage(), pView->calculateZoomPercentForWholePage()));
+		if (pFrame->getZoomPercentage() == pView->calculateZoomPercentForWholePage())
+			s = EV_MIS_Toggled;
+		break;
+	case AP_MENU_ID_VIEW_ZOOM_WIDTH:
+		xxx_UT_DEBUGMSG(("Width: %d %d\n", pFrame->getZoomPercentage(), pView->calculateZoomPercentForPageWidth()));
+		if (pFrame->getZoomPercentage() == pView->calculateZoomPercentForPageWidth())
+			s = EV_MIS_Toggled;
+		break;
+	default:
+		UT_ASSERT_NOT_REACHED ();
+	}
+
+	return s;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_Lists)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail(pView, EV_MIS_ZERO);
+
+	if(pView->getDocument()->areStylesLocked() || pView->isHdrFtrEdit())
+	{
+		return EV_MIS_Gray;
+	}
+
+	return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_MailMerge)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail(pView, EV_MIS_ZERO);
+	
+	if (0 == IE_MailMerge::getMergerCount ())
+		return EV_MIS_Gray;
+	return EV_MIS_ZERO;
+}
